;*******************************************************************************************
;   Beschreibung: FM Radio mit RDA5807M (I2C)
;   
;   Takt : 4 MHz
;
;								 *							 *
;*******************************************************************************************
;                                                                                          *
;    Dateiname:	 	FM_Tuner_P16F876A.asm						                               		 *
;    Datum:        	17.12.2021                                                           *
;    Version: 		V1                                                                  *
;                                                                                          *
;    Autor:       	Werner Hirsch                                                       *
;    Internet:      www.wernerhirsch.de                                                           *
;    e-mail:		wernerhirschhh@aol.com                                                          *
;                                                                                          *
;*******************************************************************************************
;																														 *
;	Bankumschaltungen:
;	BANK_0:	BCF STATUS,RP0
;			BCF STATUS,RP1
;	BANK_1:	BSF STATUS,RP0
;			BCF STATUS,RP1
;	BANK_2:	BCF STATUS,RP0
;			BSF STATUS,RP1
;	BANK_3:	BSF STATUS,RP0
;			BSF STATUS,RP1														 *
;to do:
;                                                                                          *
;*******************************************************************************************


	list      p=16f876a			;definiert den verwendeten Prozessor
	#include <p16f876a.inc>		;in diesem File stehen die Adressen und Bitnamen

	ERRORLEVEL      -302		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -305		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -306		;SUPPRESS BANK SELECTION MESSAGES
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_OFF & _HS_OSC & _WRT_OFF & _LVP_OFF & _DEBUG_OFF & _CPD_OFF 
	; Mit '__CONFIG' werden die Configurationsbits gesetzt, z.B. Watchdog an/aus, Debugmodus

;===========================
;  DEFINITIONEN
;===========================
#define OUT_AMP_ONOFF	PORTA, 0		;PIN RA0 (pin 2) Ausgang zum Ein/Auschalten der Endstufe
#define IN_NC_RA1		PORTA, 1		;PIN RA1 (pin 3)
#define IN_NC_RA2		PORTA, 2		;PIN RA2 (pin 4)
#define IN_NC_RA3		PORTA, 3		;PIN RA3 (pin 5)
#define IN_NC_RA4		PORTA, 4		;PIN RA4 (pin 6)
#define IN_NC_RA5		PORTA, 5		;PIN RA5 (pin 7)
#define IN_IR			PORTB, 0		;PIN RB0 (pin 21) Eingang IR-Empfnger
#define IN_ONOFF		PORTB, 1		;PIN RB1 (pin 22) Eingang On/Off
#define IN_MENUE		PORTB, 2		;PIN RB2 (pin 23) Eingang Men
#define IN_OK			PORTB, 3		;PIN RB3 (pin 24) Eingang OK
#define IN_DOWN			PORTB, 4		;PIN RB4 (pin 25) Eingang Down / Vol-
#define IN_UP			PORTB, 5		;PIN RB5 (pin 26) Eingang Up / Vol+
#define IN_LINKS		PORTB, 6		;PIN RB6 (pin 27) PGC, Eingang Links / P-
#define IN_RECHTS		PORTB, 7		;PIN RB7 (pin 28) PGD, Eingang Rechts / P+
#define IN_NC_RC0		PORTC, 0		;PIN RC0 (pin 11)
#define OUT_ONOFF		PORTC, 1		;PIN RC1 (pin 12) Ausgang On/Off
#define DISP_EN			PORTC, 2		;PIN RC2 (pin 13) Enable Display
#define SCL_IO			PORTC, 3		;PIN RC3 (pin 14) I2C Clock
#define SDA_IO			PORTC, 4		;PIN RC4 (pin 15) I2C Daten
#define DISP_RS			PORTC, 5		;PIN RC5 (pin 16) Unterscheidung Befehl/Daten
#define SI_DAT			PORTC, 6		;PIN RC6 (pin 17) Serielle Daten zum Display
#define SI_CLK			PORTC, 7		;PIN RC7 (pin 18) Serieller Takt zum Display


;===========================
;  VARIABLEN BANK 0
;===========================
w_temp        	EQU	0x7D        ; werden fr Interrupts verwendet, um den Status vor 
status_temp   	EQU	0x7E        ; der Interruptroutine zu sichern
pclath_temp		EQU	0x7F

LOOP_1			EQU 0x20		;Zhler fr Wartezeit
LOOP_2			EQU 0x21		;Zhler fr Wartezeit
SHIFT_REG		EQU	0x22		;Datum, welches an das Display gesendet werden soll
SHIFT_CNT		EQU	0x23		;Zhler fr die Datenausgabe
SENDER			EQU 0x24		;Sender Nr. (1-16)
FREQUENZ		EQU 0x25		;Frequenz
WB02H_H			EQU 0x26		;Adresse 0x02, H-Byte (Write)
WB02H_L			EQU 0x27		;Adresse 0x02, L-Byte (Write)
WB03H_H			EQU 0x28		;Adresse 0x03, H-Byte (Write)
WB03H_L			EQU 0x29		;Adresse 0x03, L-Byte (Write)
WB04H_H			EQU 0x2A		;Adresse 0x04, H-Byte (Write)
WB04H_L			EQU 0x2B		;Adresse 0x04, L-Byte (Write)
WB05H_H			EQU 0x2C		;Adresse 0x05, H-Byte (Write)
WB05H_L			EQU 0x2D		;Adresse 0x05, L-Byte (Write)
RB0AH_H			EQU 0x2E		;Adresse 0x0A, H-Byte (Read)
RB0AH_L			EQU 0x2F		;Adresse 0x0A, L-Byte (Read)
RB0BH_H			EQU 0x30		;Adresse 0x0B, H-Byte (Read)
RB0BH_L			EQU 0x31		;Adresse 0x0B, L-Byte (Read)
RB0CH_H			EQU 0x32		;Adresse 0x0C, H-Byte (Read) RDS Block A
RB0CH_L			EQU 0x33		;Adresse 0x0C, L-Byte (Read) RDS Block A
RB0DH_H			EQU 0x34		;Adresse 0x0D, H-Byte (Read) RDS Block B
RB0DH_L			EQU 0x35		;Adresse 0x0D, L-Byte (Read) RDS Block B
RB0EH_H			EQU 0x36		;Adresse 0x0E, H-Byte (Read) RDS Block C
RB0EH_L			EQU 0x37		;Adresse 0x0E, L-Byte (Read) RDS Block C
RB0FH_H			EQU 0x38		;Adresse 0x0F, H-Byte (Read) RDS Block D
RB0FH_L			EQU 0x39		;Adresse 0x0F, L-Byte (Read) RDS Block D
STAT_RADIO		EQU 0x3A		;Statusregister:	Bit0: 0=Sender, 1=Frequenz (Programm Tasten/ R/L Tasten)
								;					Bit1: 0=kein IR-Befehl, 1=neuer IR-Befehl
								;					Bit2: 0=kein IR-Fehler, 1= IR-Fehler
								;					Bit3: IR-Befehl Mute On/Off
								;					Bit4: 0=Mute Off, 1=Mute On,
								;					Bit5: 0=Display nicht gelscht, 1=Display gelscht
								;					Bit7:6: Statusbits fr Update/Freigabe Datum/Uhrzeit
MENU_COUNT		EQU 0x3B		;
MENU_COUNT_MAX	EQU 0x3C		;
MENU_STAT		EQU 0x3D		;Statusregister:	Bit0: Update Bit Menpunkte (horizontal)
								;					Bit1: 0=Vertik. Menpunkt -, 1=Vertik. Menpunkt +
								;					Bit7: Update Bit Hauptmen (vertikal)
TMR1_EXT		EQU 0x3E		;Timer 1 Erweiterung
SEK_ADJ			EQU 0x3F		;4 sek adjust Reg.
SEKUNDE			EQU 0x40		;Register interne Uhr, Sekunde
MINUTE			EQU 0x41		;Register interne Uhr, Minute
STUNDE			EQU 0x42		;Register interne Uhr, Stunde
STAT_TIMER		EQU 0x43		;Status Register Timer: Bit 0: 66ms, Bit 1: 1s, Bit 2: 4s, Bit 3: 4s fr Volume
								; Bit 4: Volumeanzeige
TEMP_11			EQU	0x44		;Allgemeines Zwischenspeicherregister
TEMP_12			EQU	0x45		;Allgemeines Zwischenspeicherregister
TEMP_13			EQU	0x46		;Allgemeines Zwischenspeicherregister
TEMP_14			EQU	0x47		;Allgemeines Zwischenspeicherregister
TEMP_15			EQU	0x48		;Allgemeines Zwischenspeicherregister
TEMP_16			EQU 0x49		;Allgemeines Zwischenspeicherregister
TEMP_17			EQU 0x4A		;Allgemeines Zwischenspeicherregister
TEMP_18			EQU	0x4B		;Allgemeines Zwischenspeicherregister
RDS_ZZ			EQU	0x4C		;RDS Zeitzone
W_TAG			EQU	0x4D		;Wochentagszahl: 0=Samstag, 1=Sonntag, ... 6=Freitag
JAHR			EQU	0x4E		;Register fr Jahr 20XX
MONAT			EQU	0x4F		;Register fr Monat
TAG				EQU	0x50		;Register fr Tag
DATUM_ZAHL_L	EQU	0x51		;Register fr Datumszahl, L_Byte (basis: 01.01.2000)
DATUM_ZAHL_H	EQU	0x52		;Register fr Datumszahl, H_Byte (basis: 01.01.2000)
STAT_DISP		EQU	0x53		;Statusregister:	Bit0: 1=Anzeige Datum/Uhrzeit
								;					Bit1: 1=Anzeige Uhrzeit
								;					Bit2: 1=Anzeige Radiotext1/2
								;					Bit3: 1=Anzeige Radiotext2
								;					Bit4: Startbedingung 1=Standby Ein
								;					Bit5: Startbedingung 1=Amp Ein 
								;					Bit6: Freigabe Anzeige Datum/Uhrzeit: 0=Aus 1=Ein
								;					Bit7: Freigabe Anzeige Radiotext: 0=Aus 1=Ein
TEXT_ADR		EQU	0x54		;Text Adresse fr Radiotext
IR_ADRESSE		EQU	0x55		;Adressregister fr RC5 IR Fernbedienung
IR_BEFEHL		EQU	0x56		;Befehlsregister fr RC5 IR Fernbedienung
IR_BIT			EQU	0x57		;Zhlregister fr RC5 IR Fernbedienung
IR_ADR_1		EQU	0x58		;Register fr 1ste Gespeicherte IR-Adresse
IR_ADR_2		EQU	0x59		;Register fr 2te Gespeicherte IR-Adresse
IR_BEF_MEN		EQU	0x5A		;Register fr Gespeicherten IR-Befehl "Menue"
IR_BEF_OK		EQU	0x5B		;Register fr Gespeicherten IR-Befehl "OK"
BEFEHL			EQU	0x5C		;Register fr Tasten-/IR-Befehle
M_HOR_CNT		EQU 0x5D		;Menpunktzhler horizontal
M_HOR_CNT_MAX	EQU 0x5E		;Menpunktzhler horizontal max
RDS_Q_MIN		EQU 0x5F		;Register fr minimales RSSI (Relative Signal Strength Indicator) fr Anzeige RDS Daten
CONTRAST		EQU 0x60		;Register fr Kontrastwert (24...63)

;===========================
;  KONSTANTE
;===========================


;========================================
;  VARIABLEN BANK 1 (Adresse 0xA0...0xEF)
;========================================



;__________________________________________________________________________________________
;
		ORG 	H'2100'			;Adresse des EEPROM fr den Brenner 
;Konstanten 
		de		0x03			;Adresse 00, Volumen 0-15
		de		0x01			;Adresse 01, Sender 1-16

		de		0x06			;Adresse 02, Frequenz S1: NDR2 87,6 MHz
		de		0x19			;Adresse 03, Frequenz S2: NDR 1 Welle Nord 89,5 MHz
		de		0x21			;Adresse 04, Frequenz S3: NDR 90,3 MHz
		de		0x40			;Adresse 05, Frequenz S4: Delta Radio 93,4 MHz
		de		0x48			;Adresse 06, Frequenz S5: N-JOY 94,2 MHz
		de		0x65			;Adresse 07, Frequenz S6: Energy 97,1 MHz
		de		0x6F			;Adresse 08, Frequenz S7: Klassik Radio 98,1 MHz
		de		0x7A			;Adresse 09, Frequenz S8: Radio3 99,2 MHz
		de		0x88			;Adresse 0A, Frequenz S9: Radio FFN 100,6 MHz
		de		0x8D			;Adresse 0B, Frequenz S10: Radio BOB 101,1 MHz
		de		0x9F			;Adresse 0C, Frequenz S11: RSH 102,9 MHz
		de		0xA6			;Adresse 0D, Frequenz S12: Radio Hamburg 103,6 MHz
		de		0xB5			;Adresse 0E, Frequenz S13: Hit-Radio Antenne Niedersachsen 105,1 MHz
		de		0xC6			;Adresse 0F, Frequenz S13: AlsterRadio 106,8 MHz
		de		0xFF			;Adresse 10, Frequenz S15: nicht belegt
		de		0xFF			;Adresse 11, Frequenz S16: nicht belegt
		de		0xFF			;Adresse 12, nicht belegt
		de		0xF0			;Adresse 13, Status fr Anzeige, Radiotext, Datum/Uhrzeit	
		de		0x10			;Adresse 14, IR-Adresse 1
		de		0x11			;Adresse 15, IR-Adresse 2
		de		0x29			;Adresse 16, IR-Befehl Men
		de		0x26			;Adresse 17, IR-Befehl OK
		de		0x59			;Adresse 18, Qmin RDS Anzeige, RSSI 0...255 entspricht 0...100%
		de		0x28			;Adresse 19, Kontrastwert: 24-63 (40)

;__________________________________________________________________________________________

		ORG     0			; Adresse, an die nach einem Reset gesprungen wird
		clrf    PCLATH          ; Rcksetzen der Page-Bits
  		goto    Start           ; Springe zum Programmanfang

		ORG     4           ; Adresse, an die nach einem Interrupt gesprungen wird
		movwf	w_temp          ; Sichere den Inhalt des W-Registers
		movf    STATUS,w        ; Verschiebe STATUS-Register ins W-Register
		movwf	status_temp     ; Sichere den Inhalt des STATUS-Registers
		movf    PCLATH,w        ; Verschiebe PCLATH-Register ins W-Register
		movwf	pclath_temp     ; Sichere den Inhalt des PCLATH-Registers


Int_serv 
	bcf STATUS,RP0			;Select BANK_0
	bcf STATUS,RP1
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;

	btfss INTCON,INTF		;prfen, ob RB0 Interrupt ausgelst ist
	goto int_timer			;nein, weiter bei timer
	call ir_routine			;Aufruf Unterprogramm zur Auswertung 
	bcf INTCON,INTF			;lsche Interrupt flag
	goto int_end

int_timer					;Sekundentimer: (3*(15*2^16+66*2^8)+1*(15*2^16+67*2^8))/4
	btfss PIR1,TMR1IF		; => (3*(983040+16896)+1*(983040+17152))/4 = 1000000 Takte
	goto int_end
	bcf PIR1,TMR1IF			;lsche Flag
	bsf STAT_TIMER,0		;setze Statusbit fr 66ms
	decfsz TMR1_EXT			;reduziere Timer1 Erweiterungsregister um 1 und prfe ob 0
	goto int_end			;Sekunde noch nicht abgelaufen, 
	bsf STAT_TIMER,1		;Sekunde abgelaufen, setze Statusbit fr 1s
	movlw d'16'				;rcksetzen des Erweiterungsregister mit Wert 16 (15+1)
	movwf TMR1_EXT
	movlw d'190'			;Rcksetzen des Timer H-Byte mit Wert 190 (256-66=190)
	movwf TMR1H
	decfsz SEK_ADJ			;reduziere Zhler um 1 und prfe ob 0
	goto $+6				;noch nicht 0, weiter
	decf TMR1H				;=0, reduziere Timer H-Byte um 1 (256-67=189)
	movlw d'4'				; und setze Zhler zurck auf Wert 4
	movwf SEK_ADJ
	bsf STAT_TIMER,2		;4 Sekunden abgelaufen, setze Statusbit fr 4s
	bcf STAT_TIMER,3		;lsche Statusbit fr 4s (Volumeanzeige)

	incf SEKUNDE			;Erhhe Sekundenzhler um 1
	movlw d'60'				;hole Wert 60 nach W
	subwf SEKUNDE,w			;und subtrahiere die Sekunden
	btfss STATUS,Z			;prfen, ob Ergebniss =0, d.h. Sekundenzhler bergelaufen
	goto int_end			;Sekundenzhler noch nicht bergelaufen
	clrf SEKUNDE			;Sekundenzhler bergelaufen, rcksetzen mit Wert 0

	incf MINUTE				;Erhhe Minutenzhler um 1
	movlw d'60'				;hole Wert 60 nach W
	subwf MINUTE,w			;und subtrahiere die Minuten
	btfss STATUS,Z			;prfen, ob Ergebniss =0, d.h. Minutenzhler bergelaufen
	goto int_end			;Minutenzhler noch nicht bergelaufen
	clrf MINUTE				;Minutenzhler bergelaufen, rcksetzen mit Wert 0

	incf STUNDE				;Erhhe Stundenzhler um 1
	movlw d'24'				;hole Wert 24 nach W
	subwf STUNDE,w			;und subtrahiere die Stunden
	btfss STATUS,Z			;prfen, ob Ergebniss =0, d.h. Stundenzhler bergelaufen
	goto int_end			;Stundenzhler noch nicht bergelaufen
	clrf STUNDE				;Stundenzhler bergelaufen, rcksetzen mit Wert 0
	incfsz DATUM_ZAHL_L		;Erhhe Datumszahl L-Byte um 1 und prfe auf berlauf
	goto $+2				;L-Byte noch nicht bergelaufen	
	incf DATUM_ZAHL_H		;L-Byte bergelaufen, Erhhe Datumszahl H-Byte um 1
	movfw DATUM_ZAHL_H		;Kopiere die Datumszahl in RB0EH_H/L Register
	movwf RB0EH_H
	movfw DATUM_ZAHL_L
	movwf RB0EH_L
	call datum_wt			;Neuberechnung von Wochentag und Datum
int_end


		movf    pclath_temp,w     	; hole die Kopie des PCLATH-Registers
		movwf	PCLATH            	; Sichere die Daten wieder zurck in das PCLATH-Register
		movf    status_temp,w     	; hole die Kopie des STATUS-Registers
		movwf	STATUS            	; Sichere die Daten wieder zurck in das STATUS-Register
		swapf   w_temp,f
		swapf   w_temp,w          	; Sichere die Daten wieder zurck in das W-Register
		retfie                    	; Rcksprung vom Interrupt
;__________________________________________________________________________________________

Start						; Beginn des Programms
	; Initialisierungen
	bcf STATUS,RP0			;Select BANK_0
	bcf STATUS,RP1
	clrf PORTA
	clrf PORTB
	clrf PORTC
	bsf OUT_AMP_ONOFF		;setze Ausgang fr "Amp Off"

	bsf STATUS,RP0			;Select BANK_1
	movlw b'00000111'		;alle Pins von Port A sind digitale Ein- oder Ausgnge
	movwf ADCON1
	movlw b'11111110'		;setze RA1 bis RA7 als Eingang und RA0 als Ausgang
	movwf TRISA
	movlw b'11111111'		;setze RB0 bis RB7 als Eingang
	movwf TRISB
	movlw b'00011001'		;setze RC0, RC3 und RC4 als Eingang, den Rest als Ausgang
	movwf TRISC
	movlw B'00001000'		;pull-up, internen Takt zhlen, bit 3: TMR0 Vorteiler 1:1
	movwf OPTION_REG
	clrf PIE1				;lsche register
	bsf PIE1,TMR1IE			;Timer 1 Interrupt erlauben
	bcf STATUS,RP0			;Select BANK_0
	movlw b'00001100'
	movwf T2CON				;Postscaler 2:1, Timer 2 on, Prescaler 1:1

	bsf OUT_ONOFF			;setze Ausgang On/Off

	movlw 0x19				;EEPROM Adresse gespeicherter Kontrastwert Anzeige (24...63)
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf CONTRAST			;Kopieren der Daten des W-Registers ins F-Register
	bsf PCLATH,3			;Select Page 1
	call Init_display		;Initialisierung des Displays
	bcf PCLATH,3			;Select Page 0
							;Initialisierung der I2C Schnittstelle zum Schreiben/Lesen des RDA5807M
	bsf STATUS, RP0			;Bank 1 
    movlw d'9'				;clock = 4000/(4*(9+1)) = 100kHz 
    movwf SSPADD			;fr I2C 
	clrf SSPCON2			;alle Bits auf 0 setzen
    bcf STATUS, RP0			;Bank 0 
	movlw B'00001000'		;master mode, clock=Fosc/(4*(SSPADD+1)) 
    movwf SSPCON			; 

	clrf TMR0
	clrf STAT_RADIO
	movlw 0xA0				;erste Adresse fr Radiotext
	movwf TEXT_ADR
	movlw b'00010000'		;bit0-2 interupt flags, bit3 RBIE disable, bit4 INTE enable, bit5 TMR0IE disable,
	movwf INTCON			;bit6 PEIE disable, bit7 GIE disable

	bsf PCLATH,3			;Select Page 1
	call disp_radio
	bcf PCLATH,3			;Select Page 0
	call clear_text			;Unterprogramm zum lschen der Register 0xA0...0xE7 und 0x1A0...0x1DF

;Registerwerte RDA5807M:
	movlw b'10100010'		;Adresse 02H:	Bit15:	0 = High impedance; 1 = Normal operation
	movwf WB02H_H			;	H-byte		Bit14:	0 = Mute; 1 = Normal operation
							;				Bit13:	0 = Stereo; 1 = Mono
							;				Bit12:	0 = Bass boost disable; 1 = enable
							;				Bit11:	0 = RCLK clock is always supply
							;				Bit10:	1 = RCLK clock use the directly input mode
							;				Bit9:	0 = Seek down; 1 = Seek up
							;				Bit8:	0 = Disable stop seek; 1 = Enable
	movlw b'00000010'		;Adresse 02H:	Bit7:	0 = wrap at the upper or lower band limit and continue seeking
	movwf WB02H_L			;	L-byte		Bit6:4:	000=32.768kHz
							;				Bit3:	0 = RDS/RBDS disable; 1 = enable
							;				Bit2:	New Demodulate Method 0=disable, 1=enable
							;				Bit1:	Soft reset 0=disable, 1=enable
							;				Bit0:	Power Up 0=disable, 1=enable
	movlw b'00000000'		;Adresse 03H:	Bit15:8: CHAN[9:2]
	movwf WB03H_H			;	H-byte
	movlw b'00000000'		;Adresse 03H:	Bit7:6:	 CHAN[1:0]
	movwf WB03H_L			;	L-byte		Bit5:	Directly Control Mode, Only used when test.
							;				Bit4:	Tune, 1=Enable, 0=Disable
							;				Bit3:2:	BAND[1:0] 00 = 87108 MHz
							;				Bit1:0:	SPACE[1:0], 00 = 100kHz
	movlw b'00001010'		;Adresse 04H:	Bit15:12: reserved =0
	movwf WB04H_H			;	H-byte		Bit11:	de-emphasis 0 = 75 s; 1 = 50 s (Europa = 50s)
							;				Bit10:	reserved =0
							;				Bit9:	softmute 0 = Disable; 1 = Enable
							;				Bit8:	reserved =0
	movlw b'00000000'		;Adresse 04H:	Bit7:0: reserved =0
	movwf WB04H_L			;	L-byte
	movlw b'10001111'		;Adresse 05H:	Bit15:	0 = 5ms interrupt; 1= interrupt until read reg0CH
	movwf WB05H_H			;	H-byte		Bit14:12: reserved =0
							;				Bit11:8: SEEK Treshhold[3:0] 1000 default
	movlw b'10000000'		;Adresse 05H:	Bit7:6:	?? 10 default
	movwf WB05H_L			;	L-byte		Bit5:4	reserved =00	
							;				Bit3:0	Volume 0000 ... 1111

;laden der Konstanten aus dem internen EEPROM in die F-Register
	movlw 0x00				;EEPROM Adresse Volume
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	addlw b'10000000'
	movwf WB05H_L			;Kopieren der Daten des W-Registers ins F-Register Volume

	movlw 0x01				;EEPROM Adresse Sendernummer (1-16)
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf SENDER			;Kopieren der Daten des W-Registers ins F-Register
	call set_frequenz

	movlw 0x13				;EEPROM Adresse Status fr Anzeige, Stationsname, Radiotext, Datum/Uhrzeit
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf STAT_DISP			;Kopieren der Daten des W-Registers ins F-Register

	movlw 0x14				;EEPROM Adresse 1ste gespeicherte IR-Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf IR_ADR_1			;Kopieren der Daten des W-Registers ins F-Register

	movlw 0x15				;EEPROM Adresse 1ste gespeicherte IR-Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf IR_ADR_2			;Kopieren der Daten des W-Registers ins F-Register

	movlw 0x16				;EEPROM Adresse gespeicherter IR-Befehl Men
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf IR_BEF_MEN		;Kopieren der Daten des W-Registers ins F-Register

	movlw 0x17				;EEPROM Adresse gespeicherter IR-Befehl OK
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf IR_BEF_OK			;Kopieren der Daten des W-Registers ins F-Register

	movlw 0x18				;EEPROM Adresse gespeichertes Qmin fr RDS Anzeige (0...255)
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf RDS_Q_MIN			;Kopieren der Daten des W-Registers ins F-Register


;RDA5807M initialisieren:
	call schreibe_i2c		;Schreibe Register nach RDA5807M
	call wait_250_p0		;Wartezeit 250ms
	bcf WB02H_L,1			;lsche Reset bit
	call schreibe_i2c		;Schreibe Register nach RDA5807M
	call wait_250_p0		;Wartezeit 250ms
	bsf WB03H_L,4			;setze Tune Bit
	call schreibe_i2c		;Schreibe Register nach RDA5807M
	call wait_250_p0		;Wartezeit 250ms
	bcf WB02H_H,5			;Stereo einschalten
	bsf WB02H_L,3			;RDS einschalten
	call schreibe_i2c

	clrf STAT_TIMER			;lsche Register
	clrf DATUM_ZAHL_L
	clrf DATUM_ZAHL_H
	clrf SEKUNDE
	clrf MINUTE
	clrf STUNDE
	clrf W_TAG
	clrf JAHR
	clrf MONAT
	clrf TAG
	clrf PIR1
	clrf T1CON
	clrf TMR1L
	movlw d'66'
	movwf TMR1H
	movlw d'16'
	movwf TMR1_EXT
	movlw d'4'
	movwf SEK_ADJ
	bsf T1CON,TMR1ON		;Timer 1 einschalten
	bsf INTCON,PEIE
	bsf INTCON,GIE
	btfsc STAT_DISP,5		;prfen, ob Startbedingung "Standby" nicht gewhlt ist
	goto standby			;Standby ist gewhlt, weiter bei Unterprogramm Standby
	btfsc STAT_DISP,4		;prfen, ob Startbedingung "Amp Ein" nicht gewhlt ist
	bcf OUT_AMP_ONOFF		;Amp Ein ist gewhlt, setze Ausgang auf Low
	bsf WB02H_L,0			;Power up enable
	call schreibe_i2c
	call wait_250_p0		;Wartezeit 250ms
	call wait_250_p0		;Wartezeit 250ms
	call wait_250_p0		;Wartezeit 250ms
	call wait_250_p0		;Wartezeit 250ms
	bsf WB02H_H,6			;Mute ausschalten
	call schreibe_i2c

;Testbereich_Anfang

;Testbereich_Ende

main						;Beginn der Hauptschleife
	call befehl				;Aufruf Unterprogramm fr Tastatur-/IR-Befehle
	btfsc BEFEHL,7			;prfen, ob IR-Befehl 0..9 Taste gedrckt wurde
	call ir_zahlen			;0..9 Taste wurde gedrckt, Aufruf Unterprogramm fr Zahlen 0..9
	btfsc BEFEHL,6			;prfen, ob Standby Taste gedrckt wurde
	goto standby			;On/Off Taste wurde gedrckt, weiter bei Unterprogramm Standby
	btfsc BEFEHL,5			;prfen, ob Down Taste gedrckt wurde
	call volume_m			;Down Taste wurde gedrckt, Aufruf unterprogramm fr Volume -
	btfsc BEFEHL,4			;prfen, ob UP Taste gedrckt wurde
	call volume_p			;UP Taste wurde gedrckt, Aufruf unterprogramm fr Volume +
	btfss BEFEHL,2			;prfen, ob Links Taste gedrckt wurde
	goto $+5				;
	btfss STAT_RADIO,0
	call sender_m
	btfsc STAT_RADIO,0
	call frequenz_m
	btfss BEFEHL,3			;prfen, ob Rechts Taste gedrckt wurde
	goto $+5
	btfss STAT_RADIO,0
	call sender_p
	btfsc STAT_RADIO,0
	call frequenz_p

	btfsc BEFEHL,0			;prfen, ob Menue Taste gedrckt wurde
	goto menue				;Menue Taste wurde gedrckt, Weiter bei Programmteil menue
	btfsc BEFEHL,1			;prfen, ob OK Taste gedrckt wurde
	call freq_sender		;Umschaltung der Rechts/Links Tasten von Sender auf Frequenz und umgekehrt	
	btfsc STAT_RADIO,3		;prfen, ob Bit fr Mute On/Off gesetzt ist
	call mute_onoff			;Unterprogramm zum setzen/lschen des Mute Bits

	btfss STAT_TIMER,0		;prfen, ob Timer 1 bergelaufen ist (65,5 ms)
	goto main
	bcf STAT_TIMER,0		;lsche Timer 1 berlaufbit
	call rds_daten			;Timer 1 bergelaufen, prfen auf neuen RDS Datensatz
	btfss STAT_TIMER,1		;prfen, ob 1s Timer bergelaufen ist
	goto main
	bcf STAT_TIMER,1		;lsche 1s Timer Bit
	bsf PCLATH,3			;Select Page 1
	call disp_p_freq_q		;Anzeige von Programm Nr., Frequenz und Qualitt
	bcf PCLATH,3			;Select Page 0	
	movfw RDS_Q_MIN			;hole Qmin nach W
	subwf RB0BH_H,w			;subtrahiere das Qualittssignal (0...255)
	btfss STATUS,C			;prfen, ob Ergebnis positiv
	goto $+6				;Ergebnis negativ, weiter
	bsf PCLATH,3			;Select Page 1
	call disp_rds_daten		;Anzeige von Station, Radiotext und Datum/Uhrzeit
	bcf PCLATH,3			;Select Page 0	
	bcf STAT_RADIO,5		;lsche Bit fr "Display gelscht"
	goto main
	btfsc STAT_RADIO,5		;prfen, ob Bit fr "Display gelscht" nicht gesetzt ist
	goto main				;Bit fr "Display gelscht" ist gesetzt, zurck zur Hauptprogrammschleife
	bsf PCLATH,3			;Select Page 1
	call clear_rdsdisp		;lsche das 2.Reihe des Display fr RDS Daten
	bcf PCLATH,3			;Select Page 0	
	bsf STAT_RADIO,5		;setze Bit fr "Display gelscht"
	goto main				;zurck zur Hauptprogrammschleife


standby						;Programmteil zum Abschalten / Einschalten
	bsf PCLATH,3			;Select Page 1
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	bcf PCLATH,3			;Select Page 0	
	bcf WB02H_H,6			;Mute einschalten
	call schreibe_i2c		;Schreibe Register nach RDA5807M
	bcf OUT_ONOFF			;lsche Ausgang On/Off
	call wait_250_p0		;Wartezeit von 250ms Page0
	call wait_250_p0		;Wartezeit von 250ms Page0
	bcf STAT_RADIO,1		;lsche Bit fr neuen IR-Befehl
	bsf OUT_AMP_ONOFF		;Amp ausschalten
	bcf WB02H_L,0			;Power up disable
	call schreibe_i2c

	call befehl				;Aufruf Unterprogramm fr Tastatur-/IR-Befehle
	btfss BEFEHL,6			;prfen, ob Standby Taste nicht gedrckt wurde
	goto $-2				;On/Off Taste wurde nicht gedrckt, zurck zur Schleife

	call wait_250_p0		;Wartezeit von 250ms Page0
	bcf STAT_RADIO,1		;lsche Bit fr neuen IR-Befehl
	bsf OUT_ONOFF			;setze Ausgang On/Off
	btfsc STAT_DISP,4		;prfen, ob Startbedingung "Amp Ein" nicht gewhlt ist
	bcf OUT_AMP_ONOFF		;Amp Ein ist gewhlt, setze Ausgang auf Low
	bsf WB02H_L,0			;Power up enable
	call schreibe_i2c
	call wait_250_p0		;Wartezeit 250ms
	call wait_250_p0		;Wartezeit 250ms
	call wait_250_p0		;Wartezeit 250ms
	call wait_250_p0		;Wartezeit 250ms
	bsf WB02H_H,6			;Mute ausschalten
	call schreibe_i2c		;Schreibe Register nach RDA5807M
	call clear_text			;Unterprogramm zum lschen der Register 0xA0...0xE7 und 0x1A0...0x1DF
	goto main				;Springe zur Hauptprogrammschleife


menue						;Programmteil Men
	movlw d'1'				;rcksetzen des Menzhlers auf1
	movwf MENU_COUNT
	bsf MENU_STAT,7			;Setze Bit fr update Men
	call wait_250_p0		;Wartezeit von 250ms Page0
	bcf STAT_RADIO,1		;lsche Bit fr neuen IR-Befehl
menue_loop
	bsf PCLATH,3			;Select Page 1
	btfsc MENU_STAT,7		;prfen des Update Bits
	call disp_menue_1		;Bit ist gesetzt, Update der Anzeige fr Hauptmen
	bcf PCLATH,3			;Select Page 0
	call befehl				;Aufruf Unterprogramm zum laden der Tasten-/IR-Befehle
	btfsc BEFEHL,5			;prfen, ob Down Taste gedrckt wurde
	call menue_dn			;Taste gedrckt, Aufruf Unterprogramm Men down
	btfsc BEFEHL,4			;prfen, ob Up Taste gedrckt wurde
	call menue_up			;Taste gedrckt, Aufruf Unterprogramm Men up
	btfsc BEFEHL,0			;prfen, ob Men Taste gedrckt wurde
	goto menue_end			;Taste gedrckt, weiter bei Men ende
	btfss BEFEHL,1			;Prfen ob "OK" Taste gedrckt ist
	goto menue_loop			;Taste nicht gedrckt, zurck zur Schleife

	bsf PCLATH,3			;Select Page 1
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	bcf PCLATH,3			;Select Page 0

	movlw d'1'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_seek_set		; Aufruf des Programmteils menu_such_set
	movlw d'2'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_savdel_set	; Aufruf des Programmteils menu_savdel_set
	movlw d'3'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_start_set		; Aufruf des Programmteils menu_start_set
	movlw d'4'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_rdsanz_set	; Aufruf des Programmteils menu_rdsanz_set
	movlw d'5'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_irtest_set	; Aufruf des Programmteils menu_irtest_set
	movlw d'6'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_iradr_set		; Aufruf des Programmteils menu_iradr_set
	movlw d'7'				;
	subwf MENU_COUNT,w	;
	btfsc STATUS, Z		
	goto menu_contr_set		; Aufruf des Programmteils menu_contr_set
	goto menue_end

menu_seek_set				;Menpunkt 1.Suchlauf, Sendersuchlauf mit Speichern der Frequenz
	bsf PCLATH,3			;Select Page 1
	call suche_start		;Aufruf des Unterprogramms zum Schreiben von "Suche Starten?"
	bcf PCLATH,3			;Select Page 0	
	call wait_250_p0		;Wartezeit von 250ms Page0
	call warte_j_n			;Unterprogramm zur Abfrage der Befehle Men (=nein) oder OK (=ja)
	call wait_250_p0		;Wartezeit von 250ms Page0
	btfss MENU_STAT,0		;prfen ob Bit fr OK gesetzt ist
	goto menue_end			;Bit ist nicht gesetzt, Abbruch
	bsf PCLATH,3			;Select Page 1
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	bcf PCLATH,3			;Select Page 0
	clrf SENDER				;Lsche Sender Register
	clrf FREQUENZ			;Lsche Frequenz Register, damit der Suchlauf bei 87,0 MHz beginnt
menu_seek_loop
	bsf WB03H_L,4			;setze Tune bit	
	bcf WB02H_H,0			;lsche seek Bit
	call freq_format		;Frequenz in die richtigen Bereiche der Schreibregister kopieren
	call schreibe_i2c		;sende die Startfrequenz von 87,0 MHz an den RDA5807M
	call wait_250_p0		;Wartezeit von 250ms Page0
	bcf WB03H_L,4			;lsche Tune bit

	bsf WB02H_H,0			;setze seek Bit
	call schreibe_i2c		;neue Einstellung an den RDA5807M senden
	btfss IN_MENUE			;prfen ob Menue Taste nicht gedrckt ist
	goto menu_seek_end		;Menue Taste ist gedrckt, Rcksprung aus dem Men
	call wait_250_p0		;Wartezeit von 250ms Page0
	call lese_i2c			;lese Register 0x0A bis 0x0F des RDA5807M
	btfss RB0AH_H,6			;prfen ob Suchvorgang beendet (Bit6)
	goto $-5				;Suchvorgang nicht beendet, zurck zur Schleife
	movfw RB0AH_L			;Suchvorgang beendet
	subwf FREQUENZ,w
	btfsc STATUS,C
	goto menu_seek_end1

	movfw RB0AH_L
	movwf FREQUENZ
	movlw d'63'				;prfen, ob Qualitt des Signals >=25%
	subwf RB0BH_H,w
	btfss STATUS,C
	goto menu_seek_loop		;Qualitt des Signals <25%, weiter suchen
	incf SENDER				;Qualitt des Signals >=25%, erhhe Sender Nr. um 1
	movlw d'17'				;prfen, ob die max. Senderanzahl (max.16) nach Erhhung berschritten ist.
	subwf SENDER,w
	btfsc STATUS,Z
	goto menu_seek_end		;Max. Senderzahl berschritten, Abbruch
	movfw RB0AH_L			;hole Frequenz nach W (0...255 => 87,0....112,5 MHz)
	movwf TEMP_13			;kopiere Frequenz in Temp Register
	movfw SENDER			;hole Sender Nr. nach W
	addlw 0x01				;addiere 1 (=> EEPROM Speicherplatz)
	call writeEE			;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht

	bsf PCLATH,3			;Select Page 1
	call disp_menu_seek		;Aufruf des Unterprogramms zur Anzeige auf dem Displays
	bcf PCLATH,3			;Select Page 0
	call wait_250_p0		;Wartezeit von 250ms Page0
	btfss IN_MENUE			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto menu_seek_end		;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife
	call wait_250_p0		;Wartezeit von 250ms Page0
	btfss IN_MENUE			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto menu_seek_end		;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife
	call wait_250_p0		;Wartezeit von 250ms Page0
	btfss IN_MENUE			;prfen ob Bit fr Befehl Menue nicht gesetzt ist
	goto menu_seek_end		;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife
	goto menu_seek_loop
menu_seek_end1
	incf SENDER				;Qualitt des Signals >=25%, erhhe Sender Nr. um 1
	movlw d'17'				;prfen, ob die max. Senderanzahl (max.16) nach Erhhung berschritten ist.
	subwf SENDER,w
	btfsc STATUS,Z
	goto menu_seek_end		;Max. Senderzahl berschritten, Abbruch	
	movlw 0xFF				;Wert fr leere Speicherpltze
	movwf TEMP_13			;kopiere Frequenz in Temp Register
	movfw SENDER			;hole Sender Nr. nach W
	addlw 0x01				;addiere 1 (=> EEPROM Speicherplatz)
	call writeEE			;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	call wait_250_p0		;Wartezeit von 250ms Page0
	goto menu_seek_end1
menu_seek_end
	bsf WB03H_L,4			;setze Tune bit
	bcf WB02H_H,0			;lsche seek Bit
	movlw 0x01				;EEPROM Adresse Sendernummer (1-16)
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf SENDER			;Kopieren der Daten des W-Registers ins F-Register
	call set_frequenz
	call schreibe_i2c
	goto menue_end

menu_savdel_set				;Menpunkt 2.Speichern, Lschen der Frequenz des aktuellen Senders
	bsf PCLATH,3			;Select Page 1
	call disp_save_delete	;Aufruf des Unterprogramms zum Schreiben von "XXX.X Speichern?" oder "FMXX Loeschen?" 
	bcf PCLATH,3			;Select Page 0	
	call wait_250_p0		;Wartezeit von 250ms Page0
	call warte_j_n			;Unterprogramm zur Abfrage der Befehle Men (=nein) oder OK (=ja)
	call wait_250_p0		;Wartezeit von 250ms Page0
	btfss MENU_STAT,0		;prfen ob Bit fr OK gesetzt ist
	goto menue_end			;Bit ist nicht gesetzt, Abbruch
	bsf PCLATH,3			;Select Page 1
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	bcf PCLATH,3			;Select Page 0
	btfss STAT_RADIO,0		;prfen, ob bit fr Frequenz gesetzt ist
	goto menu_del_set		;Bit ist nicht gesetzt =>weiter bei Lschen

menu_store_set				;Menpunkt 2a.Speichern, Speichern der Frequenz des aktuellen Senders
	movfw RB0AH_L			;kopiere Frequenz des aktuellen Senders in Frequenzregister
	movwf FREQUENZ
	movlw d'17'				;EEPROM Adresse Frequenz (2-17)
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	sublw 0xFF				;prfen ob der Speicher voll ist (Adresse 17 >0xFF<)
	btfss STATUS,Z
	goto menu_store_end1	;Speicher voll, (Adresse 17 >0xFF<)
	movlw d'16'				;prfen, ob Frequenz schon vorhanden ist
	movwf TEMP_11
	movfw TEMP_11
	addlw 0x01				;addiere Wert 1 (fr EEPROM Adresse)
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	subwf FREQUENZ,w		;subtrahiere die Frequenz in W
	btfsc STATUS,Z			;
	goto menu_store_end		;Frequenz schon vorhanden, Abbruch
	decfsz TEMP_11			;Frequenz ungleich der Frequenz im Speicher, reduziere Senderzhler um 1 und prfe ob 0
	goto $-7				;Zhler noch nicht 0, zurck zur Schleife

	movlw d'15'				;prfen, ob Frequenz schon vorhanden ist
	movwf TEMP_11
menu_store_loop
	movfw TEMP_11
	addlw 0x01				;addiere Wert 1 (fr EEPROM Adresse)
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf TEMP_13
	subwf FREQUENZ,w
	btfsc STATUS,C
	goto menu_store_end2
	movfw TEMP_11
	addlw 0x02
	call writeEE
	movlw d'50'
	call wait_p0			;Wartezeit von 50ms
	decfsz TEMP_11			;
	goto menu_store_loop	;
	goto menu_store_end		;
menu_store_end2
	movfw FREQUENZ
	movwf TEMP_13
	movfw TEMP_11
	addlw 0x02
	call writeEE
	movlw d'50'
	call wait_p0			;Wartezeit von 50ms
	goto menu_store_end
menu_store_end1
	bsf PCLATH,3			;Select Page 1
	call disp_sp_voll		;Aufruf Unterprogramm zum schreiben "Speicher voll"
	bcf PCLATH,3			;Select Page 0
	movlw d'3'				;Hinweis soll 3-4 Sek auf dem Display angezeigt werden
	movwf LOOP_1
	bcf STAT_TIMER,1		;lsche 1s Timer Bit
	btfss STAT_TIMER,1		;prfen, ob 1s Timer bergelaufen ist
	goto $-1
	decfsz LOOP_1			;reduziere Zhler und prfe, ob null
	goto $-4
menu_store_end
	bcf STAT_RADIO,0		;lsche Bit fr manuelle Frequenzeinstellung
	movlw 0x01				;EEPROM Adresse Sendernummer (1-16)
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf SENDER			;Kopieren der Daten des W-Registers ins F-Register
	call set_frequenz		;zu der Sender Nr. wird die Frequenz aus dem EEPROM geladen 
	call schreibe_i2c		; und an den RDA5807M gesendet
	goto menue_end

menu_del_set				;Menpunkt 2b.Lschen, Lschen des aktuellen Senders
	movfw SENDER			;hole Sender Nr. nach W
	addlw 0x02				;addiere Wert 2 (entspricht EEPROM Speicherplatz der Frequenz des Senders+1)
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register	
	movwf TEMP_13			;kopiere Frequenz des Senders+1 in Temp Register
	movfw SENDER			;hole Sender Nr. nach W
	addlw 0x01				;addiere Wert 1 (entspricht EEPROM Speicherplatz der Frequenz des Senders+0)
	call writeEE			;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	movlw d'50'
	call wait_p0			;Wartezeit von 50ms
	incf SENDER				;erhhe Senderzhler um 1
	movlw d'17'
	subwf SENDER,w
	btfss STATUS,Z
	goto menu_del_set
menu_del_end
	movlw 0x01				;EEPROM Adresse Sendernummer (1-16)
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf SENDER			;Kopieren der Daten des W-Registers ins F-Register
	call set_frequenz		;zu der Sender Nr. wird die Frequenz aus dem EEPROM geladen 
	call schreibe_i2c		; und an den RDA5807M gesendet
	goto menue_end


menu_start_set				;Menpunkt 3.Startbedingungen: Volume, Sender Nr., Standby 0/1, Amp 0/1
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf M_HOR_CNT
	movlw d'4'				;Anzeige hat 4 Stellen, die gesetzt werden
	movwf M_HOR_CNT_MAX		;
	movfw STAT_DISP			;sichern des Registers in Temp Register (Bit 4: Standby 0/1, Bit 5: Amp 0/1)
	movwf TEMP_11
	movfw WB05H_L			;sichern des Volume Register in Temp Register
	movwf TEMP_17
	bcf TEMP_17,7			;lsche Bit 7 im Volume Register
	movfw SENDER			;sichern der Sender Nr. in Temp Register
	movwf TEMP_18

	bsf MENU_STAT,0			;setze Statusbit
	call wait_250_p0		;Wartezeit von 250ms Page0	
	bcf STAT_RADIO,1		;lsche Statusbit fr neuen IR Befehl
	bsf PCLATH,3			;Select Page 1
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	bcf PCLATH,3			;Select Page 0
loop_start_set
	call befehl				;Aufruf Unterprogramm zum laden der Tasten-/IR-Befehle
	btfsc BEFEHL,0			;prfen ob Taster fr Befehl Menue nicht gedrckt ist
	goto menue_end			;Taste ist gedrckt, Rcksprung in die Hauptschleife	
	btfsc BEFEHL,1			;prfen ob Taster fr Befehl OK nicht gedrckt ist
	goto copy_start			;Taste ist gedrckt, gehe zum Ende des Programmteils
	btfsc BEFEHL,3			;prfen ob Taster fr Befehl "right" nicht gedrckt ist
	call inc_m_hor_cnt		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;prfen ob Taster fr Befehl "left" nicht gedrckt ist
	call dec_m_hor_cnt		;wenn ja berspringe diesen Befehl
	btfsc MENU_STAT,0		;prfen ob bit in register nicht gesetzt ist
	call dig_start_end		;wenn ja berspringe diesen Befehl
	btfss BEFEHL,4			;prfen ob Taster fr Befehl "up" gedrckt ist
	goto $+3				;Taster ist nicht gedrckt, weiter bei nchster Prfung
	bsf MENU_STAT,1			;setze Statusbit fr "up"
	call dig_start			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,5			;prfen ob Taster fr Befehl "down" nicht gedrckt ist
	call dig_start			;wenn ja berspringe diesen Befehl
	goto loop_start_set		;zurck zur Schleife
copy_start	
	bcf STAT_DISP,5			;lsche und kopiere den Status der Bits 5:4 von Temp 11 nach Stat Disp
	bcf STAT_DISP,4
	btfsc TEMP_11,5			;
	bsf STAT_DISP,5
	btfsc TEMP_11,4			;
	bsf STAT_DISP,4
	btfss STAT_DISP,5		;prfen, ob Amp eingeschaltet sein soll
	bsf OUT_AMP_ONOFF		;Bit 5 ist nicht gesetzt, Amp ausschalten
	btfsc STAT_DISP,5		;prfen, ob Amp ausgeschaltet sein soll
	bcf OUT_AMP_ONOFF		;Bit 5 ist gesetzt, Amp einschalten

	movfw STAT_DISP			;Speichern des Registers im EEPROM
	movwf TEMP_13			;zu speichernder Wert
	movlw 0x13				;EEPROM Adresse
	call writeEE
	movfw TEMP_17			;speichern des Temp Register in Volume Register
	movwf WB05H_L			;sichern des Volume Register in Temp Register
	movwf TEMP_13			;zu speichernder Wert
	movlw 0x00				;EEPROM Adresse
	call writeEE
	bsf WB05H_L,7			;setze Bit 7 im Volume Register
	movfw TEMP_18
	movwf SENDER			;sichern der Sender Nr. in Temp Register
	movwf TEMP_13			;zu speichernder Wert
	movlw 0x01				;EEPROM Adresse
	call writeEE
	movlw d'50'				;Wartezeit von 50ms
	call wait_p0
	call set_frequenz		;setzen der Frequenz zu der gewhlten Sender Nr.
	goto menue_end


menu_rdsanz_set				;Menpunkt 4.RDS Anzeige, Auswahl der Anzeige Radiotext, Datum/Uhrzeit und min Q
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf M_HOR_CNT
	movlw d'3'				;Anzeige hat 3 Stellen, die gesetzt werden
	movwf M_HOR_CNT_MAX		;
	movfw RDS_Q_MIN			;kopiere Qmin fr RDS Anzeige (0...255) in das Multiplikationsregister Temp_16:15
	movwf TEMP_15			; Qualitt = Signal x 100/256
	clrf TEMP_16
	movlw 0x64				;kopiere Wert 100 (hex 0x64) in 2tes Multiplikationsregister Temp_18:17
	movwf TEMP_17
	clrf TEMP_18
	call mul16				;Multiplikation, das durch 256 geteilte Ergebnis steht in Temp 12
	movfw STAT_DISP			;kopieren des Registers ins Temp Register
	movwf TEMP_11

	bsf MENU_STAT,0			;setze Statusbit
	call wait_250_p0		;Wartezeit von 250ms Page0	
	bcf STAT_RADIO,1		;lsche Statusbit fr neuen IR Befehl
	bsf PCLATH,3			;Select Page 1
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	bcf PCLATH,3			;Select Page 0
loop_rdsanz_set
	call befehl				;Aufruf Unterprogramm zum laden der Tasten-/IR-Befehle
	btfsc BEFEHL,0			;prfen ob Taster fr Befehl Menue nicht gedrckt ist
	goto menue_end			;Taste ist gedrckt, Rcksprung in die Hauptschleife	
	btfsc BEFEHL,1			;prfen ob Taster fr Befehl OK nicht gedrckt ist
	goto copy_rdsanz		;Taste ist gedrckt, gehe zum Ende des Programmteils
	btfsc BEFEHL,3			;prfen ob Taster fr Befehl "right" nicht gedrckt ist
	call inc_m_hor_cnt		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;prfen ob Taster fr Befehl "left" nicht gedrckt ist
	call dec_m_hor_cnt		;wenn ja berspringe diesen Befehl
	btfsc MENU_STAT,0		;prfen ob bit in register nicht gesetzt ist
	call dig_rdsanz_end		;wenn ja berspringe diesen Befehl
	btfss BEFEHL,4			;prfen ob Taster fr Befehl "up" gedrckt ist
	goto $+3				;Taster ist nicht gedrckt, weiter bei nchster Prfung
	bsf MENU_STAT,1			;setze Statusbit fr "up"
	call dig_rdsanz			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,5			;prfen ob Taster fr Befehl "down" nicht gedrckt ist
	call dig_rdsanz			;wenn ja berspringe diesen Befehl
	goto loop_rdsanz_set	;zurck zur Schleife
copy_rdsanz	
	bcf STAT_DISP,7			;lsche und kopiere den Status der Bits 7:6 von Temp 11 nach Stat Disp
	bcf STAT_DISP,6
	btfsc TEMP_11,7			;
	bsf STAT_DISP,7
	btfsc TEMP_11,6			;
	bsf STAT_DISP,6
	movfw STAT_DISP			;Speichern des Registers im EEPROM
	movwf TEMP_13			;zu speichernder Wert
	movlw 0x13				;EEPROM Adresse
	call writeEE

	clrf TEMP_13			;Signal = Qmin x 256/100, lsche die Divisionsregister Temp 11 und Temp 13
	clrf TEMP_11			;der Qmin Wert (%) x 256 steht schon in Temp 12
	movlw d'100'			;kopiere Wert 100 in 2tes Divisionsregister
	movwf TEMP_15
	clrf TEMP_16
	call div16				;Division: TEMP_13:11 / TEMP_16:15 = TEMP_13:11, Rest = TEMP_18:17
	movfw TEMP_11			;Speichern von Qmin im EEPROM
	movwf TEMP_13			;zu speichernder Wert
	movlw 0x18				;EEPROM Adresse
	call writeEE
	goto menue_end


menu_irtest_set				;Menpunkt 5: In diesem Programmteil werden die RC5 IR-Adresse und IR-Befehle angezeigt
	bsf STAT_RADIO,2		;setze IR Fehlerbit fr Start anzeige
	bsf PCLATH,3			;Select Page 1
	call disp_set_irtest
	bcf PCLATH,3			;Select Page 0
loop_irtest_set
	bsf PCLATH,3			;Select Page 1
	btfsc STAT_RADIO,1		;prfen, ob Statusbit fr IR-Befehl nicht gesetzt ist
	call disp_set_irtest	;Bit ist gesetzt, Aufruf des Unterprogramms
	btfsc STAT_RADIO,2		;prfen, ob Statusbit fr IR-Fehler nicht gesetzt ist
	call disp_set_irtest	;Bit ist gesetzt, Aufruf des Unterprogramms
	bcf PCLATH,3			;Select Page 0
	btfss IN_MENUE			;prfen ob Men Taste nicht gedrckt ist (high ist)
	goto menue_end			;Taste ist gedrckt, gehe zum Ende des Programmteiles
	btfss IN_OK				;prfen ob OK Taste nicht gedrckt ist (high ist)
	goto menue_end			;Taste ist gedrckt, gehe zum Ende des Programmteiles
	goto loop_irtest_set	


menu_iradr_set				;Menpunkt 6: Setzen der RC5 IR-Adressen/Befehle
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf M_HOR_CNT
	movlw d'4'				;Anzeige hat 4 Stellen, die gesetzt werden
	movwf M_HOR_CNT_MAX		;
	movfw IR_ADR_1			;kopieren der 1.RC5-Adresse ins Temp Register
	movwf RB0EH_H
	movfw IR_ADR_2			;kopieren der 2.RC5-Adresse ins Temp Register
	movwf RB0EH_L
	movfw IR_BEF_MEN		;kopieren des Befehls fr Men ins Temp Register
	movwf RB0FH_H
	movfw IR_BEF_OK			;kopieren des Befehls fr OK ins Temp Register
	movwf RB0FH_L
	bsf MENU_STAT,0			;setze Statusbit
	call wait_250_p0		;Wartezeit von 250ms Page0	
	bsf PCLATH,3			;Select Page 1
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	bcf PCLATH,3			;Select Page 0
loop_iradr_set
	btfss IN_MENUE			;prfen ob Taster fr Befehl Menue nicht gedrckt ist
	goto copy_iradr1		;Taste ist gedrckt, Rcksprung in die Hauptschleife	
	btfss IN_OK				;prfen ob Taster fr Befehl OK nicht gedrckt ist
	goto copy_iradr			;Taste ist gedrckt, gehe zum Ende des Programmteils
	btfss IN_RECHTS			;prfen ob Taster fr Befehl "right" nicht gedrckt ist
	call inc_m_hor_cnt		;wenn ja berspringe diesen Befehl
	btfss IN_LINKS			;prfen ob Taster fr Befehl "left" nicht gedrckt ist
	call dec_m_hor_cnt		;wenn ja berspringe diesen Befehl
	btfsc MENU_STAT,0		;prfen ob bit in register nicht gesetzt ist
	call dig_iradrbef_end	;wenn ja berspringe diesen Befehl
	btfsc IN_UP				;prfen ob Taster fr Befehl "up" gedrckt ist
	goto $+3				;Taster ist nicht gedrckt, weiter bei nchster Prfung
	bsf MENU_STAT,1			;setze Statusbit fr "up"
	call dig_iradrbef		;wenn ja berspringe diesen Befehl
	btfss IN_DOWN			;prfen ob Taster fr Befehl "down" nicht gedrckt ist
	call dig_iradrbef		;wenn ja berspringe diesen Befehl
	goto loop_iradr_set		;zurck zur Schleife
copy_iradr
	movfw IR_ADR_1			;Speichern der 1sten RC5-Adresse im EEPROM
	movwf TEMP_13			;zu speichernder Wert
	movlw 0x14				;EEPROM Adresse
	call writeEE
	movfw IR_ADR_2			;Speichern der 2ten RC5-Adresse im EEPROM
	movwf TEMP_13			;zu speichernder Wert
	movlw 0x15				;EEPROM Adresse
	call writeEE
	movfw IR_BEF_MEN		;Speichern des Befehls fr Men im EEPROM
	movwf TEMP_13			;zu speichernder Wert
	movlw 0x16				;EEPROM Adresse
	call writeEE
	movfw IR_BEF_OK			;Speichern des Befehls fr OK im EEPROM
	movwf TEMP_13			;zu speichernder Wert
	movlw 0x17				;EEPROM Adresse
	call writeEE
	goto menue_end
copy_iradr1
	movfw RB0EH_H			;Rcksichern der Temp Register in F Register
	movwf IR_ADR_1
	movfw RB0EH_L			;
	movwf IR_ADR_2
	movfw RB0FH_H			;
	movwf IR_BEF_MEN
	movfw RB0FH_L			;
	movwf IR_BEF_OK
	goto menue_end

menu_contr_set				;Menpunkt 7: Einstellen des Kontrast fr das Display
	movfw CONTRAST			;Kontrastwert in Temp Register sichern
	movwf TEMP_11
	bsf PCLATH,3			;Select Page 1
	call disp_set_contrast	;Anzeigen des Kontrast Wertes
	bcf PCLATH,3			;Select Page 0
	call wait_250_p0		;Wartezeit von 250ms Page0
contr_set_loop
	call befehl				;Aufruf Unterprogramm zum laden der Tasten-/IR-Befehle
	btfsc BEFEHL,0			;prfen ob Taster fr Befehl Menue nicht gedrckt ist
	goto contr_set_end1
	btfsc BEFEHL,1			;prfen ob Taster fr Befehl OK nicht gedrckt ist
	goto contr_set_end
	btfsc BEFEHL,4			;prfen, ob Up Taste nicht gedrckt ist
	call inc_contrast
	btfsc BEFEHL,5			;prfen, ob Down Taste nicht gedrckt ist
	call dec_contrast
	btfsc BEFEHL,3			;prfen, ob Rechts Taste nicht gedrckt ist
	call inc_contrast
	btfsc BEFEHL,2			;prfen, ob links Taste nicht gedrckt ist
	call dec_contrast
	goto contr_set_loop
contr_set_end				;Einstellungen sollen bernommen werden
	movfw CONTRAST			;Speichern des Kontrast Wertes im internen EEPROM
	movwf TEMP_13
	movlw 0x19				;EEPROM Adresse
	call writeEE
	goto menue_end
contr_set_end1				;Einstellungen sollen nicht bernommen werden
	movfw TEMP_11			;Rcksichern des in Temp Register gespeicherten Kontrast Wertes
	movwf CONTRAST
	goto menue_end


menue_end					;Standardroutine fr Beendigung des Mens
	clrf MENU_STAT			;Lsche Register
	movlw d'1'				;Rcksetzen des vertikalen Menpunktzhlers auf 1
	movwf M_HOR_CNT
	call wait_250_p0		;Wartezeit von 250ms Page0
	bcf STAT_RADIO,1		;lsche Bit fr neuen IR-Befehl
	bsf PCLATH,3			;Select Page 1
	call cursor_off
	call clear_display
	bcf PCLATH,3			;Select Page 0
	goto main

;*******************************************************************************************
;	Unterprogramme Mensteuerung
;*******************************************************************************************

menue_up
	incf MENU_COUNT			;erhhe Menpunkt Zhler um 1
	movlw d'8'				;Maximalwert der Menpunkte +1
	subwf MENU_COUNT,w		;subtrahiere aktuellen Menpunkt nach Erhhung vom Maximalwert und prfe ob 0
	btfss STATUS, Z		
	goto menue_updn_end		;Ergebnis ist nicht 0
	movlw d'1'				;Ergebnis ist 0, Rcksetzen des Zhlers mit Wert 1
	movwf MENU_COUNT
	goto menue_updn_end
menue_dn
	decfsz MENU_COUNT		;reduziere Menpunkt Zhler um 1 und prfe ob 0
	goto menue_updn_end		;Zhler ist nicht 0
	movlw d'7'				;Zhler ist 0, Rcksetzen des Zhlers mit Maximalwert
	movwf MENU_COUNT
menue_updn_end
	call wait_250_p0		;Wartezeit von 250ms Page0
	bcf STAT_RADIO,1		;lsche Bit fr neuen IR-Befehl
	bsf MENU_STAT,7			;setze das Statusbit
	return

inc_m_hor_cnt				;Unterprogramm zur Erhhung des Menpunktes horizontal
	movfw M_HOR_CNT_MAX		;
	subwf M_HOR_CNT,w
	btfss STATUS, Z			;prfen ob Zhler gleich dem Maximalwert ist
	goto $+4				;wenn ja, berspringe diesen Befehl	
	movlw d'1'
	movwf M_HOR_CNT
	goto inc_hor_cnt_end
	incf M_HOR_CNT
inc_hor_cnt_end
	bsf MENU_STAT,0			;setze das Status bit
	return		

dec_m_hor_cnt				;Unterprogramm zur Reduzierung des Menpunktes horizontal
	decfsz M_HOR_CNT		;reduziere Zhler um 1 und prfe ob der der Zhler 0 ist
	goto dec_hor_cnt_end	;wenn ja, berspringe diesen Befehl	
	movfw M_HOR_CNT_MAX		;
	movwf M_HOR_CNT
dec_hor_cnt_end
	bsf MENU_STAT,0			;setze das Status bit
	return

warte_j_n					;Unterprogramm zur Abfrage der Befehle Men (=nein) oder OK (=ja)
	call befehl				;Aufruf Unterprogramm zum laden der Tasten-/IR-Befehle
	btfsc BEFEHL,0			;prfen, ob Men Taste gedrckt wurde
	goto $+5				;Taste gedrckt, weiter bei lschen des Statusbit
	btfss BEFEHL,1			;Prfen ob "OK" Taste gedrckt ist
	goto warte_j_n			;Taste nicht gedrckt, zurck zur Schleife
	bsf MENU_STAT,0			;setze Statusbit
	return
	bcf MENU_STAT,0			;lsche Statusbit
	return

inc_contrast				;Unterprogramm zur Erhhung des Kontrastwertes (Menpunkt 7.)
	incf CONTRAST
	movlw d'64'
	subwf CONTRAST,w
	btfss STATUS,C
	goto incdec_contr_end
	movlw d'63'
	movwf CONTRAST
	goto incdec_contr_end
dec_contrast				;Unterprogramm zur Reduzierung des Kontrastwertes (Menpunkt 7.)
	decf CONTRAST
	movlw d'30'
	subwf CONTRAST,w
	btfsc STATUS,C
	goto incdec_contr_end
	movlw d'30'
	movwf CONTRAST
incdec_contr_end
	bsf PCLATH,3			;Select Page 1
	call Init_display		;Unterprogramm zur Initialisierung des Display
	call disp_set_contrast	;Anzeigen des Kontrast Wertes
	bcf PCLATH,3			;Select Page 0
	call wait_250_p0		;Wartezeit von 250ms Page0
	return


dig_start					;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt 3
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_start2			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movfw TEMP_17			;hole den Wert fr Volume nach W
	call inc_dec_dig		;erhhe / reduziere den Wert
	movwf TEMP_17			;rcksichern des genderten Wertes in Temp Register
	goto dig_start_end
dig_start2
	movlw d'2'				;prfen ob der 2. Wert verndert werden soll,
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_start3			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	decf TEMP_18
	movfw TEMP_18			;hole den Wert fr Sender Nr. nach W
	call inc_dec_dig		;erhhe / reduziere den Wert
	movwf TEMP_18			;rcksichern des genderten Wertes in Temp Register
	incf TEMP_18
	goto dig_start_end
dig_start3
	movlw d'3'				;prfen ob der 3. Wert verndert werden soll,
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_start4			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_11,5			;prfen, ob Bit fr Anzeige Radiotext gesetzt ist
	goto $+3				;Bit ist nicht gesetzt, setze Bit
	bcf TEMP_11,5			;Bit ist gesetzt, lsche Bit
	goto dig_start_end
	bsf TEMP_11,5			;Bit ist nicht gesetzt, setze Bit
	goto dig_start_end
dig_start4
	movlw d'4'				;prfen ob der 2. Wert verndert werden soll,
	subwf M_HOR_CNT,w		;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_start_end		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_11,4			;prfen, ob Bit fr Anzeige Datum/Uhrzeit gesetzt ist
	goto $+3				;Bit ist nicht gesetzt, setze Bit
	bcf TEMP_11,4			;Bit ist gesetzt, lsche Bit
	goto dig_start_end
	bsf TEMP_11,4			;Bit ist nicht gesetzt, setze Bit
dig_start_end
	bsf PCLATH,3			;Select Page 1
	call disp_set_start		;Anzeige Menpunkt 3
	bcf PCLATH,3			;Select Page 0
	movlw 0x07				;vorbereiten fuer Curser Tabelle im Bereich 0x0700..0x07FF
	movwf PCLATH			;
	movfw M_HOR_CNT			;
	addlw 0x07
	call cursor_tab
	bsf PCLATH,3			;Select Page 1
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bcf PCLATH,3			;Select Page 0
	call wait_250_p0		;Wartezeit von 250ms Page0
	bcf MENU_STAT,0			;lsche die Status bits
	bcf MENU_STAT,1
	return

inc_dec_dig
	movwf TEMP_13
	btfss MENU_STAT,1		;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_start			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf TEMP_13			;Bit ist gesetzt, erhhe Zhler um 1
	movlw d'16'				;lade Maximalwert+1 ins W-Register
	subwf TEMP_13,w			;subtrahiere Wert nach Erhhung
	btfss STATUS,Z			;prfen ob Ergebnis =0
 	goto dec_start_end		;
	clrf TEMP_13			;Rcksetzen des Registers mit Wert 0
	goto dec_start_end		;
dec_start
	movfw TEMP_13			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;Wert ist 0, Rcksetzen des Zhlers
	decf TEMP_13			;Wert ist noch nicht 0, reduziere Zhler um 1
	goto dec_start_end		;
	movlw d'15'				;Rcksetzen des Zhlers mit dem Maximalwert
	movwf TEMP_13
dec_start_end
	movfw TEMP_13			;hole den Wert nach W
	return


dig_rdsanz					;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt 4
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_rdsanz2		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_11,7			;prfen, ob Bit fr Anzeige Radiotext gesetzt ist
	goto $+3				;Bit ist nicht gesetzt, setze Bit
	bcf TEMP_11,7			;Bit ist gesetzt, lsche Bit
	goto dig_rdsanz_end
	bsf TEMP_11,7			;Bit ist nicht gesetzt, setze Bit
	goto dig_rdsanz_end
dig_rdsanz2
	movlw d'2'				;prfen ob der 2. Wert verndert werden soll,
	subwf M_HOR_CNT,w		;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_rdsanz3		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_11,6			;prfen, ob Bit fr Anzeige Datum/Uhrzeit gesetzt ist
	goto $+3				;Bit ist nicht gesetzt, setze Bit
	bcf TEMP_11,6			;Bit ist gesetzt, lsche Bit
	goto dig_rdsanz_end
	bsf TEMP_11,6			;Bit ist nicht gesetzt, setze Bit
	goto dig_rdsanz_end
dig_rdsanz3
	movlw d'3'				;prfen ob der 1. Wert verndert werden soll,
	subwf M_HOR_CNT,w		;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_rdsanz_end		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss MENU_STAT,1		;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_rdsanz3		;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf TEMP_12			;Bit ist gesetzt, erhhe Zhler um 1
	movlw d'100'			;lade Maximalwert+1 ins W-Register
	subwf TEMP_12,w			;subtrahiere Wert nach Erhhung
	btfss STATUS,Z			;prfen ob Ergebnis =0
 	goto dig_rdsanz_end	;
	clrf TEMP_12			;Rcksetzen des Registers mit Wert 0
	goto dig_rdsanz_end
dec_rdsanz3
	movfw TEMP_12			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;Wert ist 0, Rcksetzen des Zhlers
	decf TEMP_12			;Wert ist noch nicht 0, reduziere Zhler um 1
	goto dig_rdsanz_end	;
	movlw d'99'				;Rcksetzen des Zhlers mit dem Maximalwert
	movwf TEMP_12
 	goto dig_rdsanz_end	;
dig_rdsanz_end
	bsf PCLATH,3			;Select Page 1
	call disp_set_rdsanz	;Anzeige Menpunkt 4
	bcf PCLATH,3			;Select Page 0
	movlw 0x07				;vorbereiten fuer Curser Tabelle im Bereich 0x0700..0x07FF
	movwf PCLATH			;
	movfw M_HOR_CNT			;
	call cursor_tab
	bsf PCLATH,3			;Select Page 1
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bcf PCLATH,3			;Select Page 0
	call wait_250_p0		;Wartezeit von 250ms Page0
	bcf MENU_STAT,0			;lsche die Status bits
	bcf MENU_STAT,1
	return


dig_iradrbef				;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt 7
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_iradrbef2		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss MENU_STAT,1		;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_iradrbef1		;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf IR_ADR_1			;Bit ist gesetzt, erhhe Zhler um 1
	movlw d'32'				;lade Maximalwert+1 ins W-Register
	subwf IR_ADR_1,w		;subtrahiere Wert nach Erhhung
	btfss STATUS,Z			;prfen ob Ergebnis =0
 	goto dig_iradrbef_end	;
	clrf IR_ADR_1			;Rcksetzen des Registers mit Wert 0
	goto dig_iradrbef_end
dec_iradrbef1
	movfw IR_ADR_1			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;Wert ist 0, Rcksetzen des Zhlers
	decf IR_ADR_1			;Wert ist noch nicht 0, reduziere Zhler um 1
	goto dig_iradrbef_end	;
	movlw d'31'				;Rcksetzen des Zhlers mit dem Maximalwert
	movwf IR_ADR_1
 	goto dig_iradrbef_end	;
dig_iradrbef2
	movlw d'2'				;prfen ob der 2. Wert verndert werden soll,
	subwf M_HOR_CNT,w		;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_iradrbef3		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss MENU_STAT,1		;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_iradrbef2		;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf IR_ADR_2			;Bit ist gesetzt, erhhe Zhler um 1
	movlw d'32'				;lade Maximalwert+1 ins W-Register
	subwf IR_ADR_2,w		;subtrahiere Wert nach Erhhung
	btfss STATUS,Z			;prfen ob Ergebnis =0
 	goto dig_iradrbef_end	;
	clrf IR_ADR_2			;Rcksetzen des Registers mit Wert 0
	goto dig_iradrbef_end
dec_iradrbef2
	movfw IR_ADR_2			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;Wert ist 0, Rcksetzen des Zhlers
	decf IR_ADR_2			;Wert ist noch nicht 0, reduziere Zhler um 1
	goto dig_iradrbef_end	;
	movlw d'31'				;Rcksetzen des Zhlers mit dem Maximalwert
	movwf IR_ADR_2
 	goto dig_iradrbef_end	;
dig_iradrbef3
	movlw d'3'				;prfen ob der 1. Wert verndert werden soll,
	subwf M_HOR_CNT,w		;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_iradrbef4		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss MENU_STAT,1		;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_iradrbef3		;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf IR_BEF_MEN			;Bit ist gesetzt, erhhe Zhler um 1
	movlw d'64'				;lade Maximalwert+1 ins W-Register
	subwf IR_BEF_MEN,w		;subtrahiere Wert nach Erhhung
	btfss STATUS,Z			;prfen ob Ergebnis =0
 	goto dig_iradrbef_end	;
	clrf IR_BEF_MEN			;Rcksetzen des Registers mit Wert 0
	goto dig_iradrbef_end
dec_iradrbef3
	movfw IR_BEF_MEN		;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;Wert ist 0, Rcksetzen des Zhlers
	decf IR_BEF_MEN			;Wert ist noch nicht 0, reduziere Zhler um 1
	goto dig_iradrbef_end	;
	movlw d'63'				;Rcksetzen des Zhlers mit dem Maximalwert
	movwf IR_BEF_MEN
 	goto dig_iradrbef_end	;
dig_iradrbef4
	movlw d'4'				;prfen ob der 1. Wert verndert werden soll,
	subwf M_HOR_CNT,w		;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_iradrbef_end	;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss MENU_STAT,1		;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_iradrbef4		;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf IR_BEF_OK			;Bit ist gesetzt, erhhe Zhler um 1
	movlw d'64'				;lade Maximalwert+1 ins W-Register
	subwf IR_BEF_OK,w		;subtrahiere Wert nach Erhhung
	btfss STATUS,Z			;prfen ob Ergebnis =0
 	goto dig_iradrbef_end	;
	clrf IR_BEF_OK			;Rcksetzen des Registers mit Wert 0
	goto dig_iradrbef_end
dec_iradrbef4
	movfw IR_BEF_OK			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;Wert ist 0, Rcksetzen des Zhlers
	decf IR_BEF_OK			;Wert ist noch nicht 0, reduziere Zhler um 1
	goto dig_iradrbef_end	;
	movlw d'63'				;Rcksetzen des Zhlers mit dem Maximalwert
	movwf IR_BEF_OK
dig_iradrbef_end
	bsf PCLATH,3			;Select Page 1
	call disp_set_iradr		;Anzeige Menpunkt 7
	bcf PCLATH,3			;Select Page 0
	movlw 0x07				;vorbereiten fuer Curser Tabelle im Bereich 0x0700..0x07FF
	movwf PCLATH			;
	movfw M_HOR_CNT			;
	addlw 0x03
	call cursor_tab
	bsf PCLATH,3			;Select Page 1
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bcf PCLATH,3			;Select Page 0
	call wait_250_p0		;Wartezeit von 250ms Page0
	bcf MENU_STAT,0			;lsche die Status bits
	bcf MENU_STAT,1
	return

;*******************************************************************************************
;	Unterprogramme zum Auslesen der RDS Daten
;*******************************************************************************************

rds_daten
	call lese_i2c			;Lese Register 0x0A bis 0x0F des RDA5807M
	btfss RB0AH_H,7			;prfen ob neuer RDS Datensatz vorliegt (Bit7)
	return					;kein neuer Datensatz, zurck zum Hauptprogramm
	btfsc RB0BH_L,0			;prfen ob Fehler in Block B vorliegen (Bit1:0)
	return					;Fehler, zurck zum Hauptprogramm
	btfsc RB0BH_L,1			;prfen ob Fehler in Block B vorliegen (Bit1:0)
	return					;Fehler, zurck zum Hauptprogramm
	btfsc RB0BH_L,2			;prfen ob Fehler in Block A vorliegen (Bit3:2)
	return					;Fehler, zurck zum Hauptprogramm
	btfsc RB0BH_L,3			;prfen ob Fehler in Block A vorliegen (Bit3:2)
	return					;Fehler, zurck zum Hauptprogramm
	movfw RB0DH_H			;hole das H-byte von Block B nach W, die Bits 7:4 geben die Art der Daten 
	movwf TEMP_13			;  an, die gesendet werden: z.B. 0010 (dez.=2) Radiotext
	swapf TEMP_13			;vertausche die oberen und unteren 4 Bit und lsche die oberen 4 Bit
	bcf TEMP_13,7
	bcf TEMP_13,6
	bcf TEMP_13,5
	bcf TEMP_13,4

	movfw TEMP_13			;prfen, ob Wert =0, d.h. Stationsname
	btfss STATUS,Z
	goto radiotext

stat_name					;Programmteil zum kopieren der 8 Zeichen des Stationsnamen in die Register 0xE0...0xE7
	bcf RB0DH_L,7			;Die Bits 3:0 geben die Position der 2Byte aus Block D an.
	bcf RB0DH_L,6			;Die Position kann 8, 9, 10 oder 15 betragen
	bcf RB0DH_L,5
	bcf RB0DH_L,4
	bcf STATUS,IRP			;lsche Bit
	movlw d'8'				;prfen, ob die Position =8
	subwf RB0DH_L,w
	btfss STATUS,Z
	goto $+4				;Position >8<, weiter bei nchster Abfrage
	movlw 0xE0				;Position =8, kopiere die F-Reg. Speicheradresse ins FSR register
	movwf FSR
	goto stat_name_end		;Weiter
	movlw d'9'				;prfen, ob die Position =9
	subwf RB0DH_L,w
	btfss STATUS,Z
	goto $+4				;Position >9<, weiter bei nchster Abfrage
	movlw 0xE2				;Position =9, kopiere die F-Reg. Speicheradresse ins FSR register
	movwf FSR
	goto stat_name_end		;Weiter
	movlw d'10'				;prfen, ob die Position =10
	subwf RB0DH_L,w
	btfss STATUS,Z
	goto $+4				;Position >10<, weiter bei nchster Abfrage
	movlw 0xE4				;Position =10, kopiere die F-Reg. Speicheradresse ins FSR register
	movwf FSR
	goto stat_name_end		;Weiter
	movlw d'15'				;prfen, ob die Position =15
	subwf RB0DH_L,w
	btfss STATUS,Z
	return					;Position >15<, Abbruch
	movlw 0xE6				;Position =15, kopiere die F-Reg. Speicheradresse ins FSR register
	movwf FSR
stat_name_end
	movfw RB0FH_H
	movwf INDF
	incf FSR
	movfw RB0FH_L
	movwf INDF
	return

radiotext
	movlw 0x02				;prfen, ob Wert =2, d.h. Radiotext
	subwf TEMP_13,w
	btfss STATUS,Z
	goto datum_uz			;Wert ist nicht 2, weiter bei Prfung Datum/Uhrzeit
	bcf RB0DH_L,7			;Die Bits 4:0 geben die Position der 4Byte aus Block C/D an.
	bcf RB0DH_L,6
	bcf RB0DH_L,5

	bcf STATUS,IRP
	btfsc RB0DH_L,4
	bsf STATUS,IRP

	bcf RB0DH_L,4

	bcf STATUS,C			;lsche Carry Bit, damit es beim rotieren nicht strt
	rlf RB0DH_L				;mit 4 multiplizieren (2x nach links schieben)
	bcf STATUS,C			;lsche Carry Bit, damit es beim rotieren nicht strt
	rlf RB0DH_L				;mit 4 multiplizieren (2x nach links schieben)	
	movfw RB0DH_L			;Adresse Bank 1
	addlw 0xA0
	movwf FSR
	movfw RB0EH_H
	movwf INDF
	incf FSR
	movfw RB0EH_L
	movwf INDF
	incf FSR
	movfw RB0FH_H
	movwf INDF
	incf FSR
	movfw RB0FH_L
	movwf INDF
	return

datum_uz
	movlw 0x04				;prfen, ob Wert =4, d.h. Datum/Uhrzeit
	subwf TEMP_13,w
	btfss STATUS,Z
	return

	clrf RDS_ZZ				;Daten Register RB0DH_L bis RB0FH_L (Block B L-Byte,Block C H/L-Byte,Block D H/L-Byte):
	rrf RB0DH_L				;	xxxxxxDD DDDDDDDD DDDDDDDH HHHHMMMM MMZVVVVV
	rrf RB0EH_H				;D=Datumszahl, H=Stunden, M=Minuten, Z=Vorzeichen Zeitversatz zu UTC
	rrf RB0EH_L				;V=Zeitversatz in 30Min, (30min = 1, 60min = 2 usw.)
	rrf RB0FH_H
	rrf RB0FH_L				;Nach Rotation steht die Datumszahl in Registern RB0DH_L bis RB0EH_L
	rrf RDS_ZZ				;=> xxxxxxxD DDDDDDDD DDDDDDDD HHHHHMMM MMMZVVVV V0000000 

	rrf RB0FH_H				;die Register RB0FH_H, RB0FH_L, RDS_ZZ werden 3x nach rechts geschoben
	rrf RB0FH_L 
	rrf RDS_ZZ
	rrf RB0FH_H
	rrf RB0FH_L 
	rrf RDS_ZZ
	rrf RB0FH_H
	rrf RB0FH_L 			;Register RB0FH_H, RB0FH_L, RDS_ZZ nach 3x schieben:
	rrf RDS_ZZ				;=> 000HHHHH MMMMMMZV VVVV0000

	rrf RB0FH_L 			;die Register RB0FH_L, RDS_ZZ werden 2x nach rechts geschoben
	rrf RDS_ZZ
	rrf RB0FH_L  			;Register RB0FH_L, RDS_ZZ nach 2x schieben:
	rrf RDS_ZZ				;=> 00MMMMMM ZVVVVV00

	rrf RDS_ZZ 				;Register RDS_ZZ wird 2x nach rechts geschoben:
	rrf RDS_ZZ 				;=> 00ZVVVVV
	rrf RDS_ZZ				;=> 000ZVVVV V: Es werden nur ganze Stunden bei dem Zeitversatz zu UTC bercksichtigt

	movlw 0x01
	btfss RB0DH_L,0
	clrw
	movwf TEMP_13			;kopiere die aktuelle Datumszahl in die Subtraktionsregister TEMP_13:11
	movfw RB0EH_H	
	movwf TEMP_12
	movfw RB0EH_L	
	movwf TEMP_11
	movlw 0xC9				;kopiere die Datumszahl fr den 01.01.2000 (51544 dez => 0xC958 hex)
	movwf TEMP_18			; in die Subtraktionsregister TEMP_18:17
	movlw 0x58	
	movwf TEMP_17
	call sub16				;subtraktion
	movfw TEMP_12			;Rcksichern der neuen Datumszahl (basis: 01.01.2000) in RB0EH Registern
	movwf RB0EH_H
	movfw TEMP_11
	movwf RB0EH_L
								;korrektur Uhrzeit/Datumszahl
	btfsc RDS_ZZ,4				;prfen, ob das Vorzeichen + (0) oder - (1) ist
	goto t_korr_minus			;Vorzeichen ist minus, weiter bei minus Korrektur
	movfw RDS_ZZ				;Vorzeichen ist plus, addiere die Stunden der Zeitverschiebung zu UTC Stunden
	addwf RB0FH_H
	movlw d'24'					;prfen ob Stundenregister bergelaufen ist
	subwf RB0FH_H,w				;subtrahiere 24 von dem Stundenregister in W
	btfss STATUS,C				;wenn Ergebniss >=0 (C=1): korrektur erforderlich, wenn Ergebniss <0 (C=0): keine korrektur
	goto t_korr_end				;Ergebniss <0 (C=0): keine korrektur
	movwf RB0FH_H				;kopiere W ins Stundenregister 
	incfsz RB0EH_L				;Erhhe L-Byte der Datumszahl um 1 und prfe auf berlauf
	goto t_korr_end				;kein berlauf, weiter
	incf RB0EH_H				;berlauf vorhanden, Erhhe H-Byte der Datumszahl um 1
	goto t_korr_end
t_korr_minus
	bcf RDS_ZZ,4				;lsche Vorzeichen Bit
	movfw RDS_ZZ				;Vorzeichen ist plus, addiere die Stunden der Zeitverschiebung zu UTC Stunden
	subwf RB0FH_H
	btfsc STATUS,C				;wenn Ergebniss >=0 (C=1): korrektur erforderlich, wenn Ergebniss <0 (C=0): keine korrektur
	goto t_korr_end				;Ergebniss <0 (C=0): keine korrektur
	movlw d'24'
	addwf RB0FH_H
	movlw 0x01
	subwf RB0EH_L
	btfsc STATUS,C
	goto t_korr_end
	decf RB0EH_H
t_korr_end
	call test_datuz				;Unterprogramm zum prfen, ob Datum/Uhrzeit bereinstimmt, wenn ja Freigabe ber STAT_RADIO,7
	btfss STAT_RADIO,7			;prfen auf bereinstimmung
	goto $+3
	btfsc STAT_RADIO,6			;
	goto update_datuz			;Stat radio 6+7 sind gesetzt, update
	btfss STAT_RADIO,6			;
	goto update_datuz			;Stat radio 6+7 sind nicht gesetzt, erstmaliges update
	bsf STAT_RADIO,7			;setze Bit 7 
	return
update_datuz
	bcf INTCON,GIE				;globale Interrupts verbieten
	clrf TMR1L					;Timer 1 zurcksetzen
	movlw d'66'
	movwf TMR1H
	movlw d'16'
	movwf TMR1_EXT
	movlw d'4'
	movwf SEK_ADJ
	clrf SEKUNDE			;Sekunden auf Null setzen
	movfw RB0EH_H			;
	movwf DATUM_ZAHL_H
	movfw RB0EH_L			;
	movwf DATUM_ZAHL_L
	movfw RB0FH_H			;
	movwf STUNDE
	movfw RB0FH_L			;
	movwf MINUTE
	bsf INTCON,GIE			;globale Interrupts erlauben	
datum_wt
	movlw 0x07				;Ermittlung des Wochentags
	movwf TEMP_15
	clrf TEMP_16
	movfw RB0EH_L
	movwf TEMP_11
	movfw RB0EH_H
	movwf TEMP_12
	clrf TEMP_13
	call div16
	movfw TEMP_17			;Kopiere die Wochentagszahl ins Wochentagszahlregister:
	movwf W_TAG				; 0=Samstag, 1=Sonntag, ... 6=Freitag
datum_rds
	clrf TEMP_13			;kopiere die aktuelle Datumszahl in die Subtraktionsregister TEMP_13:11
	movfw RB0EH_H	
	movwf TEMP_12
	movfw RB0EH_L	
	movwf TEMP_11
	movlw 0x05				;kopiere die Tagesanzahl fr 4 Jahre (1461 dez => 0x05B5 hex)
	movwf TEMP_18			; in die Subtraktionsregister TEMP_18:17
	movlw 0xB5	
	movwf TEMP_17
	clrf JAHR				;lsche Register fr Jahr
	movlw 0x04				;addiere 4 zum Jahresregister
	addwf JAHR
	call sub16				;subtraktion
	call test_zero			;wenn Ergebnis der Subtraktion =0: Status,C =1
	btfsc STATUS,C			;prfen, ob Ergebnis der Subtraktion >0< (Status,C =0)
	goto monat_end1			;Ergebnis der Subtraktion =0: Status,C =1, weiter bei 
	btfss TEMP_13,7			;prfen, ob Ergebnis der Subtraktion <0 (bit7 =1)
	goto $-7				;Ergebnis der Subtraktion >0, zurck zur Schleife
	call add16				;Ergebnis <=0, letzte Subtraktion rckgngig machen
	movlw 0x04				;subtrahiere 4 vom Jahresregister
	subwf JAHR
	movlw 0x01				;kopiere die Tagesanzahl fr 1 Schaltjahr (366 dez => 0x016E hex)
	movwf TEMP_18			; in die Subtraktionsregister TEMP_18:17
	movlw 0x6E	
	movwf TEMP_17
	incf JAHR				;erhhe Jahresregister um 1
	call sub16				;subtraktion
	call test_zero			;wenn Ergebnis der Subtraktion =0: Status,C =1
	btfsc STATUS,C			;prfen, ob Ergebnis der Subtraktion >0< (Status,C =0)
	goto monat_end1			;Ergebnis der Subtraktion =0: Status,C =1, weiter bei 
	btfss TEMP_13,7			;prfen, ob Ergebnis der Subtraktion <0 (bit7 =1)
	goto $+4				;Ergebnis der Subtraktion ist positiv (Bit7 =Low), weiter bei Subtraktion 365 Tage
	call add16				;Ergebnis negativ (Bit7 =High), letzte Subtraktion rckgngig machen
	decf JAHR				;reduziere Jahresregister um 1
	goto $+0x0B
	decf TEMP_17
	incf JAHR				;erhhe Jahresregister um 1
	call sub16				;subtraktion
	call test_zero			;wenn Ergebnis der Subtraktion =0: Status,C =1
	btfsc STATUS,C			;prfen, ob Ergebnis der Subtraktion >0< (Status,C =0)
	goto monat_end1			;Ergebnis der Subtraktion =0: Status,C =1, weiter bei 
	btfss TEMP_13,7			;prfen, ob Ergebnis der Subtraktion <0 (bit7 =1)
	goto $-6				;Ergebnis der Subtraktion ist positiv (Bit7 =Low), zurck zur Schleife
	call add16				;Ergebnis negativ (Bit7 =High), letzte Subtraktion rckgngig machen
	decf JAHR				;reduziere Jahresregister um 1
	
	movlw 0x01
	movwf MONAT
	clrf TEMP_18
	movlw d'31'				;Monat Jan
	call monat_zahl
	btfss STATUS,C			;prfen, ob
	goto monat_end
	movlw d'28'				;Monat Feb	
	btfsc JAHR,0
	goto $+3
	btfss JAHR,1
	movlw d'29'				;
	call monat_zahl
	btfss STATUS,C			;prfen, ob
	goto monat_end
	movlw d'31'				;Monat Mar
	call monat_zahl	
	btfss STATUS,C			;prfen, ob
	goto monat_end
	movlw d'30'				;Monat Apr
	call monat_zahl	
	btfss STATUS,C			;prfen, ob
	goto monat_end
	movlw d'31'				;Monat Mai
	call monat_zahl	
	btfss STATUS,C			;prfen, ob
	goto monat_end
	movlw d'30'				;Monat Jun
	call monat_zahl	
	btfss STATUS,C			;prfen, ob
	goto monat_end
	movlw d'31'				;Monat Jul
	call monat_zahl	
	btfss STATUS,C			;prfen, ob
	goto monat_end
	movlw d'31'				;Monat Aug
	call monat_zahl	
	btfss STATUS,C			;prfen, ob
	goto monat_end
	movlw d'30'				;Monat Sep
	call monat_zahl	
	btfss STATUS,C			;prfen, ob
	goto monat_end
	movlw d'31'				;Monat Okt
	call monat_zahl	
	btfss STATUS,C			;prfen, ob
	goto monat_end	
	movlw d'30'				;Monat Nov
	call monat_zahl	
monat_end 
	movfw TEMP_11
	movwf TAG
	incf TAG
	return	
monat_end1
	movlw 0x01
	movwf TAG
	movwf MONAT
	return	

monat_zahl
	movwf TEMP_17			
	incf MONAT				;erhhe Monatsregister um 1
	call sub16				;subtraktion
	btfsc TEMP_13,7
	goto $+3				;Ergebnis <=0
	bsf STATUS,C			;Ergebnis >0
	return					;Ergebnis der Subtraktion >0 , Rcksprung
	call add16				;Ergebnis <=0 , letzte Subtraktion rckgngig machen
	decf MONAT				;reduziere Jahresregister um 1
	bcf STATUS,C
	return

test_zero				;prfen, ob Ergebnis der Subtraktion =0 (Register TEMP_13:11 = 0)
	bcf STATUS,C			;lsche Bit
	movfw TEMP_13			;hole Temp_13 nach W
	btfss STATUS,Z			;prfen ob =0
	return					;>0<, Rcksprung
	movfw TEMP_12			;hole Temp_12 nach W
	btfss STATUS,Z			;prfen ob =0
	return					;>0<, Rcksprung
	movfw TEMP_11			;hole Temp_11 nach W
	btfss STATUS,Z			;prfen ob =0
	return					;>0<, Rcksprung
	bsf STATUS,C			;alle Register =0, setze Bit C
	return

test_datuz					;test, ob neues RDS Datum /Uhrzeit mit internem Datum/Uhrzeit bereinstimmt
	movfw RB0EH_H			;prfen, ob RDS Datumszahl mit interner Datumszahl bereinstimmt, H-Byte
	subwf DATUM_ZAHL_H,w	;
	btfss STATUS,Z
	goto datuz_fehler		;keine bereinstimmung, weiter bei Fehler

	movfw RB0EH_L			;prfen, ob RDS Datumszahl mit interner Datumszahl bereinstimmt, L-Byte
	subwf DATUM_ZAHL_L,w
	btfss STATUS,Z
	goto datuz_fehler		;keine bereinstimmung, weiter bei Fehler

	movfw RB0FH_H			;prfen, ob RDS Stunde mit interner Stunde bereinstimmt, L-Byte
	subwf STUNDE,
	btfss STATUS,Z
	goto datuz_fehler		;keine bereinstimmung, weiter bei Fehler
	btfsc STAT_RADIO,7		;prfen ob Bit 7 nicht gesetzt ist
	bsf STAT_RADIO,6		;Bit 7 ist gesetzt, setze auch Bit 6 fr 2malige bereinstimmung
	bsf STAT_RADIO,7		;bereinstimmung, gebe Anzeige Datum/Uhrzeit frei
	return
datuz_fehler
	bcf STAT_RADIO,7		;keine bereinstimmung, sperre Anzeige Datum/Uhrzeit
	return

;*******************************************************************************************
;	Unterprogramme: Volume +/-, Sender/Frequenz +/-, Mute on/off und weitere
;*******************************************************************************************

volume_m
	bcf WB05H_L,7
	movfw WB05H_L
	btfsc STATUS,Z
	goto volume_end1
	decf WB05H_L
	goto volume_end	
volume_p
	bcf WB05H_L,7
	movlw d'15'
	subwf WB05H_L,w
	btfsc STATUS,Z	
	goto volume_end1
	incf WB05H_L
volume_end
	bsf WB05H_L,7
	call schreibe_i2c
volume_end1
	bsf WB05H_L,7
	bsf PCLATH,3		;Select Page 1
	call disp_volume	;Anzeige des aktuellen Volumes (0...15)
	bcf PCLATH,3		;Select Page 0	
	call wait_250_p0	;Unterprogramm fr Wartezeit 250ms Page0
	bcf STAT_RADIO,1	;lsche Bit fr neuen IR-Befehl
	bsf STAT_TIMER,3	;setze 4 s Timer Bits 3+4
	bsf STAT_TIMER,4
	return

freq_sender				;Umschaltung der Rechts/Links Tasten von Sender auf Frequenz und umgekehrt
	btfss STAT_RADIO,0
	goto $+3
	bcf STAT_RADIO,0
	goto $+2
	bsf STAT_RADIO,0
	call wait_250_p0	;Unterprogramm fr Wartezeit 250ms Page0
	return


sender_m
	decfsz SENDER		;reduziere Senderzhler um 1 und prfe ob 0
	goto sender_end		;noch nicht 0
	movlw 0x01			;=0, rcksetzen des Zhlers mit Wert 1
	movwf SENDER
	goto sender_end
sender_p
	incf SENDER			;erhhe Senderzhler um 1
	movlw d'17'
	subwf SENDER,w
	btfss STATUS,Z
	goto sender_end
	movlw d'16'
	movwf SENDER
sender_end
	call set_frequenz
	call schreibe_i2c
	call wait_250_p0	;Unterprogramm fr Wartezeit 250ms Page0
	call wait_250_p0	;Unterprogramm fr Wartezeit 250ms Page0
	call clear_text		;Unterprogramm zum lschen der Register 0xA0...0xE7 und 0x1A0...0x1DF
	bsf PCLATH,3		;Select Page 1
	call clear_display
	bcf PCLATH,3		;Select Page 0	
	bcf STAT_RADIO,1	;lsche Bit fr neuen IR-Befehl
	return


frequenz_m
	movlw 0x01
	subwf FREQUENZ
	btfsc STATUS,C
	goto frequenz_end
	clrf FREQUENZ
	goto frequenz_end
frequenz_p
	incfsz FREQUENZ
	goto frequenz_end
	movlw 0xFF
	movwf FREQUENZ
frequenz_end
	call freq_format
	call schreibe_i2c
	call wait_250_p0	;Unterprogramm fr Wartezeit 250ms Page0
	call clear_text		;Unterprogramm zum lschen der Register 0xA0...0xE7 und 0x1A0...0x1DF
	bsf PCLATH,3		;Select Page 1
	call clear_display
	bcf PCLATH,3		;Select Page 0	
	bcf STAT_RADIO,1	;lsche Bit fr neuen IR-Befehl
	return


mute_onoff
	btfss STAT_RADIO,4		;prfen, ob Bit fr Mute On gesetzt ist
	goto $+4				;Bit ist nicht gesetzt, weiter
	bcf STAT_RADIO,4		;Bit ist gesetzt, lsche Bit
	bsf WB02H_H,6			;Mute ausschalten
	goto $+3
	bsf STAT_RADIO,4		;Bit ist nicht gesetzt, setze Bit
	bcf WB02H_H,6			;Mute einschalten
	call schreibe_i2c		;Schreibe Register nach RDA5807M
	call wait_250_p0		;Unterprogramm fr Wartezeit 250ms Page0
	bcf STAT_RADIO,1		;lsche Bit fr neuen IR-Befehl
	bcf STAT_RADIO,3		;lsche Bit fr IR-Befehl Mute On/Off
	return

read_data
	call wait_250_p0		;Unterprogramm fr Wartezeit 250ms Page0
	call lese_i2c
	return

set_frequenz
	movfw SENDER
	addlw 0x01
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf FREQUENZ			;Kopieren der Daten des W-Registers ins F-Register Frequenz
	call freq_format		;Unterprogramm zum Einstellen des Senders der Sendernummer
	return

freq_format
	movfw FREQUENZ
	movwf WB03H_H
	bcf WB03H_L,7			;lsche Bits 7:6
	bcf WB03H_L,6
	bcf STATUS,C			;lsche Carry Bit, damit es beim rotieren nicht strt
	rrf WB03H_H
	btfsc STATUS,C			;prfen, ob ausgeschobenes Bit =0
	bsf WB03H_L,6			;ausgeschobenes Bit =1, setze Bit
	bcf STATUS,C			;lsche Carry Bit, damit es beim rotieren nicht strt
	rrf WB03H_H
	btfsc STATUS,C			;prfen, ob ausgeschobenes Bit =0
	bsf WB03H_L,7			;ausgeschobenes Bit =1, setze Bit
	return

befehl						;Unterprogramm zum Laden der Tasten-/IR-befehle
	clrf BEFEHL				;lsche Register fr Tasten-/IR-befehle
	btfss IN_MENUE			;prfen,ob Men Taste nicht gedrckt ist
	bsf BEFEHL,0			;Taste gedrckt, setze Bit 0
	btfss IN_OK				;prfen,ob OK Taste nicht gedrckt ist
	bsf BEFEHL,1			;Taste gedrckt, setze Bit 1
	btfss IN_LINKS			;prfen,ob Links Taste nicht gedrckt ist
	bsf BEFEHL,2			;Taste gedrckt, setze Bit 2
	btfss IN_RECHTS			;prfen,ob Rechts Taste nicht gedrckt ist
	bsf BEFEHL,3			;Taste gedrckt, setze Bit 3
	btfss IN_UP				;prfen,ob Up Taste nicht gedrckt ist
	bsf BEFEHL,4			;Taste gedrckt, setze Bit 4
	btfss IN_DOWN			;prfen,ob Down Taste nicht gedrckt ist
	bsf BEFEHL,5			;Taste gedrckt, setze Bit 5
	btfss IN_ONOFF			;prfen,ob Ein/Aus Taste nicht gedrckt ist
	bsf BEFEHL,6			;Taste gedrckt, setze Bit 6

	btfss STAT_RADIO,1		;prfen, ob neuer IR-Befehl anliegt
	return					;kein neuer IR-Befehl, Rcksprung

	movfw IR_ADR_1			;prfen, ob die empfangene IR-Adresse der 1sten gespeicherten Adresse entspricht
	subwf IR_ADRESSE,w
	btfsc STATUS,Z
	goto $+5				;empfangene IR-Adresse entspricht der gespeicherten, weiter
	movfw IR_ADR_2			;prfen, ob die empfangene IR-Adresse der 2ten gespeicherten Adresse entspricht
	subwf IR_ADRESSE,w
	btfss STATUS,Z
	return					;empfangene IR-Adresse entspricht nicht den gespeicherten, Abbruch

	movlw d'10'				;prfen, ob der IR_Befehl einer Zahl von 0...9 oder einem Befehl entspricht
	subwf IR_BEFEHL,w
	btfss STATUS,C
	bsf BEFEHL,7			;Ergebnis negativ, d.h. IR_Befehl <10, setze Bit fr Zahl von 0..9

	movfw IR_BEF_MEN		;= Befehl Men
	subwf IR_BEFEHL,w		;subtrahiere den empfangenen IR-Befehl
	btfsc STATUS,Z			;prfen ob Ergebnis 0
	bsf BEFEHL,0			;Ergebnis = 0, setze Bit fr Men

	movfw IR_BEF_OK			;= Befehl OK
	subwf IR_BEFEHL,w		;subtrahiere den empfangenen IR-Befehl
	btfsc STATUS,Z			;prfen ob Ergebnis 0
	bsf BEFEHL,1			;Ergebnis = 0, setze Bit fr OK

	movlw d'16'				;= Befehl Volumen + => im Men "up"
	subwf IR_BEFEHL,w		;subtrahiere den empfangenen IR-Befehl
	btfsc STATUS,Z			;prfen ob Ergebnis 0
	bsf BEFEHL,4			;Ergebnis = 0, setze Bit fr "up"

	movlw d'17'				;= Befehl Volumen - => im Men "down"
	subwf IR_BEFEHL,w		;subtrahiere den empfangenen IR-Befehl
	btfsc STATUS,Z			;prfen ob Ergebnis 0
	bsf BEFEHL,5			;Ergebnis = 0, setze Bit fr "down"

	movlw d'32'				;= Befehl Kanal + => im Men "right"
	subwf IR_BEFEHL,w		;subtrahiere den empfangenen IR-Befehl
	btfsc STATUS,Z			;prfen ob Ergebnis 0
	bsf BEFEHL,3			;Ergebnis = 0, setze Bit fr "right"

	movlw d'33'				;= Befehl Kanal - => im Men "left"
	subwf IR_BEFEHL,w		;subtrahiere den empfangenen IR-Befehl
	btfsc STATUS,Z			;prfen ob Ergebnis 0
	bsf BEFEHL,2			;Ergebnis = 0, setze Bit fr "left"

	movlw d'12'				;= Befehl System On/Off
	subwf IR_BEFEHL,w		;subtrahiere den empfangenen IR-Befehl
	btfsc STATUS,Z			;prfen ob Ergebnis 0
	bsf BEFEHL,6			;Ergebnis = 0, setze Bit fr "System On/Off"

	movlw d'13'				;= Befehl Mute On/Off
	subwf IR_BEFEHL,w		;subtrahiere den empfangenen IR-Befehl
	btfsc STATUS,Z			;prfen ob Ergebnis 0
	bsf STAT_RADIO,3		;Ergebnis = 0, setze Bit fr "Mute On/Off"

	call wait_250_p0		;Unterprogramm fr Wartezeit 250ms Page0
	bcf STAT_RADIO,1		;lsche Statusbit fr neuen IR Befehl
	return

ir_zahlen
	bcf BEFEHL,7			;lsche Status Bit fr IR-Zahl
	movfw IR_BEFEHL			;kopiere die IR-Zahl (0...9) in RB0AH_L Register
	movwf RB0AH_L
	clrf RB0AH_H			;lsche RB0AH_H Register
	bsf PCLATH,3			;Select Page 1
	call disp_ir_zahl		;Unterprogramm zur Anzeige der IR-Zahlen
	bcf PCLATH,3			;Select Page 0
	movlw d'8'				;Maximale Wartezeit = 8 x 250ms = 2 Sek 
	movwf TEMP_13
ir_zahlen1
	call befehl				;Unterprogramm zum Laden der Tasten-/IR-befehle
	btfsc BEFEHL,7			;prfen, ob Status Bit fr IR-Zahl nicht gesetzt ist
	goto $+5
	call wait_250_p0		;Unterprogramm fr Wartezeit 250ms Page0
	decfsz TEMP_13
	goto ir_zahlen1
	goto ir_zahl_ausw		;
	bcf BEFEHL,7			;lsche Status Bit fr IR-Zahl	
	call mul_x10
	bsf PCLATH,3			;Select Page 1
	call disp_ir_zahl		;Unterprogramm zur Anzeige der IR-Zahlen
	bcf PCLATH,3			;Select Page 0
	btfss STAT_RADIO,0		;prfen, ob Bit fr Frequenz gesetzt ist (4 Stellen)
	goto ir_zahl_ausw		;Bit fr Frequenz ist nicht gesetzt, d.h. Programm (2 Stellen)
	movlw d'8'				;Maximale Wartezeit = 8 x 250ms = 2 Sek 
	movwf TEMP_13
ir_zahlen2
	call befehl				;Unterprogramm zum Laden der Tasten-/IR-befehle
	btfsc BEFEHL,7			;prfen, ob Status Bit fr IR-Zahl nicht gesetzt ist
	goto $+5
	call wait_250_p0		;Unterprogramm fr Wartezeit 250ms Page0
	decfsz TEMP_13
	goto ir_zahlen2
	goto ir_zahl_ausw		;
	bcf BEFEHL,7			;lsche Status Bit fr IR-Zahl	
	call mul_x10
	bsf PCLATH,3			;Select Page 1
	call disp_ir_zahl		;Unterprogramm zur Anzeige der IR-Zahlen
	bcf PCLATH,3			;Select Page 0
	movlw d'8'				;Maximale Wartezeit = 8 x 250ms = 2 Sek 
	movwf TEMP_13
ir_zahlen3
	call befehl				;Unterprogramm zum Laden der Tasten-/IR-befehle
	btfsc BEFEHL,7			;prfen, ob Status Bit fr IR-Zahl nicht gesetzt ist
	goto $+5
	call wait_250_p0		;Unterprogramm fr Wartezeit 250ms Page0
	decfsz TEMP_13
	goto ir_zahlen3
	goto ir_zahl_ausw		;
	bcf BEFEHL,7			;lsche Status Bit fr IR-Zahl	
	call mul_x10
	bsf PCLATH,3			;Select Page 1
	call disp_ir_zahl		;Unterprogramm zur Anzeige der IR-Zahlen
	bcf PCLATH,3			;Select Page 0
ir_zahl_ausw
	call wait_250_p0		;Unterprogramm fr Wartezeit 250ms Page0
	btfss STAT_RADIO,0		;prfen, ob Bit fr Frequenz gesetzt ist (4 Stellen)
	goto ir_zahl_pgrm		;Bit ist nicht gesetzt => Auswertung Stations Nr.
	movfw RB0AH_L			;kopiere L/H-Byte in Subtraktionsregister
	movwf TEMP_11
	movfw RB0AH_H			;kopiere L/H-Byte in Subtraktionsregister
	movwf TEMP_12
	clrf TEMP_13
	movlw 0x66				;kopiere Wert 870 (hex: 0x000366) in die Subtraktionsregister TEMP_18:17
	movwf TEMP_17
	movlw 0x03
	movwf TEMP_18	
	call sub16				;16 bit Subtraktion: TEMP_13:11 - TEMP_18:17 = TEMP_13:11
	btfsc TEMP_13,7			;prfen, ob eingegebene Frequenz >=87,0MHz
	return					;eingegebene Frequenz <87,0MHz => Abbruch
	movfw TEMP_12			;prfen, ob eingegebene Frequenz <=112,5MHz
	btfss STATUS,Z
	return					;eingegebene Frequenz >112,5MHz => Abbruch
	movfw TEMP_11
	movwf FREQUENZ
	call freq_format	
	goto ir_zahlen_end
ir_zahl_pgrm				;
	movfw RB0AH_L			;prfen, ob L-Byte zwischen 1 und 16 liegt, hole L-Byte nach W
	btfsc STATUS,Z			;prfen, ob L-Byte >0<
	return					;L-Byte =0 => Abbruch
	sublw d'16'				;subtrahiere L-Byte von 16
	btfss STATUS,C			;prfen, ob Ergebnis positiv ist
	return					;Ergebnis ist negativ, d.h. L-Byte >16 (max. Programm Nr.) => Abbruch
	movfw RB0AH_L			;kopiere L-Byte nach Sender Nr.
	movwf SENDER
	call set_frequenz
ir_zahlen_end
	call schreibe_i2c
	call wait_250_p0	;Unterprogramm fr Wartezeit 250ms Page0
	call clear_text		;Unterprogramm zum lschen der Register 0xA0...0xE7 und 0x1A0...0x1DF
	bsf PCLATH,3		;Select Page 1
	call clear_display
	bcf PCLATH,3		;Select Page 0	
	bcf STAT_RADIO,1	;lsche Bit fr neuen IR-Befehl
	return
	

mul_x10						;Unterprogramm zum Multiplizieren der Register RB0AH_H:L mit 10
	bcf STATUS,C			;lsche Carry Bit, damit es beim rotieren nicht strt
	rlf RB0AH_L				;RB0AH_H:L 3x nach links schiebe, x2		
	rlf RB0AH_H
	movfw RB0AH_L			;sichern der Multiplikation x2 in Temp 16:15 Registern
	movwf TEMP_15
	movfw RB0AH_H
	movwf TEMP_16
	rlf RB0AH_L				;x4		
	rlf RB0AH_H
	rlf RB0AH_L				;x8		
	rlf RB0AH_H
	movfw TEMP_15			;addiere L-Byte (x2) zu L-Byte (x8)
	addwf RB0AH_L
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incf RB0AH_H			;es ist ein bertrag vorhanden und das L+1 Byte von Zahl2 wird um 1 erhht
	movfw TEMP_16			;addiere L-Byte (x2) zu L-Byte (x8)
	addwf RB0AH_H
	movfw IR_BEFEHL			;addiere IR-Zahl zu vorhergehender IR-Zahl (x10)
	addwf RB0AH_L
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incf RB0AH_H			;es ist ein bertrag vorhanden und das L+1 Byte von Zahl2 wird um 1 erhht
	return

clear_text					;Unterprogramm zum lschen der Register 0xA0 bis 0xE7 (Bank1) und 0x1A0 bis 0x1DF (Bank3)
	bcf STATUS,IRP			;lsche IRP Bit (indirekte Adressierung Bank0/1)
	movlw 0xA0				;lade Erste Adresse in FSR Register
	movwf FSR
	movlw 0xE8				;maximale adresse +1
	movwf LOOP_1
	call clear_text_loop

	bsf STATUS,IRP			;setze IRP Bit (indirekte Adressierung Bank2/3)
	movlw 0xA0				;lade Erste Adresse in FSR Register
	movwf FSR
	movlw 0xE0				;maximale adresse +1
	movwf LOOP_1
	call clear_text_loop

	clrf RB0CH_H			;alle Register sind gelscht. Jetzt noch die 
	clrf RB0CH_L			;	Register der RDS Daten Block A-D lschen
	clrf RB0DH_H			;
	clrf RB0DH_L			;
	clrf RB0EH_H			;
	clrf RB0EH_L			;
	clrf RB0FH_H			;
	clrf RB0FH_L
	return					;fertig

clear_text_loop
	clrf INDF				;lsche INDF Register
	incf FSR,F 				;Erhhe FSR Register
	movfw LOOP_1			;maximale adresse +1
	subwf FSR,w				;subtrahiere FSR in w
	btfss STATUS,Z			;prfen, ob 0, d.h. maximale adresse +1 erreicht ist
	goto clear_text_loop	;maximale adresse +1 noch nicht erreicht, zurck zur Schleife
	return



wait_250_p0					;Unterprogramm fr Wartezeit 250ms Page0
	movlw d'250'
	call wait_p0
	return

wait_p0
	movwf LOOP_1
loop_250_p0
	movlw d'250'			;loop_10 mu 4 Takten entsprechen fr 1ms: 4MHz /4/250/4Takte
	movwf LOOP_2
loop_10_p0
	nop
	decfsz LOOP_2
	goto loop_10_p0
	decfsz LOOP_1
	goto loop_250_p0
	return

;*******************************************************************************************
;	Unterprogramme zum Lesen/Schreiben des internen EEPROM
;*******************************************************************************************
	
readEE							;Unterprogramm zum lesen der Zelle deren Adresse in W steht
	bsf STATUS, RP1 			;EEADR liegt in der Bank 2 
	bcf STATUS, RP0
 	movwf EEADR 				;schreibe die Adresse in EEADR 
	bsf STATUS, RP0				;Bank 3
	bcf EECON1, EEPGD			;Access data memory
	bsf EECON1, RD 				;EEPROM Leseproze starten 
	bcf STATUS, RP0 			;Bank 2 
	movf EEDATA, W 				;Die Daten der EEPROM Zelle nach W kopieren 
	bcf STATUS, RP1 			;Bank 0 
	return

writeEE							;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	bsf STATUS, RP0				;Bank 3 (EECON1 ist in Bank 3)
	bsf STATUS, RP1
	btfsc EECON1, WR			;prfen, ob das EEPROM beschrieben werden kann
	goto $-1					;warten, bis das bit WR von der Hardware zurckgesetzt wurde
	bcf STATUS, RP0				;Bank 2 (EEADR und EEDATA sind in Bank 2)
	movwf EEADR					;Die Adresse an die die Daten geschrieben werden sollen ist im W-Register
	bcf STATUS, RP1				;Bank 0
	movfw TEMP_13				;hole zu speichernden Wert ins W-Register
	bsf STATUS, RP1				;Bank 2 (EEADR und EEDATA sind in Bank 2)
	movwf EEDATA				;Wert wird in EEADR geschrieben.
	bsf STATUS, RP0				;Bank 3
	bcf EECON1, EEPGD			;Das Bit EEPGD lschen, um das interne Daten-EEPROM auszuwhlen. Wird das Bit gesetzt, wird der Progamm-Flash ausgewhlt
	bsf EECON1, WREN			;erlaubt das Beschreiben des Daten-EEPROM
	bcf INTCON, GIE				;falls Interrupts verwendet werde, mssen diese abgeschaltet werden	
	movlw 0x55					;Um das EEPROM zu beschreiben, mssen die folgenden 5 Befehle
	movwf EECON2				;genau in dieser Reihenfolge aufgerufen werden.
	movlw 0xAA					;
	movwf EECON2				;
	bsf EECON1, WR				;Jetzt beginnt der eigentliche Schreibvorgang.
	bsf INTCON, GIE				;Die Interrupts knnen jetzt wieder freigegeben werden.
	bcf EECON1, WREN			;Das Beschreiben des EEPROM verbieten.
	bcf STATUS, RP0				;Bank 0
	bcf STATUS, RP1
	return						;Zurck ins Hauptprogramm

;*******************************************************************************************
;	Unterprogramme zum Lesen/Schreiben des RDA5807M ber I2C Bus
;*******************************************************************************************

lese_i2c
	call i2c_on				;Bus aktiv 
	movlw 0x21				;Device Adresse: 0010000*, Lesen: bit 0 =1
	call i2c_tx				;zum Schreiben adressieren 

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	movwf RB0AH_H			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

  	call i2c_rx 
	movfw SSPBUF			; I2C Empfangsregister auslesen 
	movwf RB0AH_L			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	movwf RB0BH_H			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

  	call i2c_rx 
	movfw SSPBUF			; I2C Empfangsregister auslesen 
	movwf RB0BH_L			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	movwf RB0CH_H			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

  	call i2c_rx 
	movfw SSPBUF			; I2C Empfangsregister auslesen 
	movwf RB0CH_L			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	movwf RB0DH_H			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

  	call i2c_rx 
	movfw SSPBUF			; I2C Empfangsregister auslesen 
	movwf RB0DH_L			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	movwf RB0EH_H			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

  	call i2c_rx 
	movfw SSPBUF			; I2C Empfangsregister auslesen 
	movwf RB0EH_L			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	movwf RB0FH_H			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

  	call i2c_rx 
	movfw SSPBUF			; I2C Empfangsregister auslesen 
	movwf RB0FH_L			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_no_ack

	call i2c_off			; Bus freigeben
	return



schreibe_i2c
	call i2c_on				;Bus aktiv 
	movlw 0x20				;Device Adresse: 0010000*, Schreiben: bit 0 =0
	call i2c_tx				;zum Schreiben adressieren 

	movfw WB02H_H				;Adr.: 0x02, H-byte: Wert der gespeichert werden soll 
	call i2c_tx 

	movfw WB02H_L				;Adr.: 0x02, L-byte: Wert der gespeichert werden soll 
	call i2c_tx 

	movfw WB03H_H				;Adr.: 0x03, H-byte: Wert der gespeichert werden soll 
	call i2c_tx 

	movfw WB03H_L				;Adr.: 0x03, L-byte: Wert der gespeichert werden soll 
	call i2c_tx 

	movfw WB04H_H				;Adr.: 0x04, H-byte: Wert der gespeichert werden soll 
	call i2c_tx 

	movfw WB04H_L				;Adr.: 0x04, L-byte: Wert der gespeichert werden soll 
	call i2c_tx 

	movfw WB05H_H				;Adr.: 0x05, H-byte: Wert der gespeichert werden soll 
	call i2c_tx 

	movfw WB05H_L				;Adr.: 0x05, L-byte: Wert der gespeichert werden soll 
	call i2c_tx 

	movlw b'00000000'			;Adr.: 0x06, H-byte
	call i2c_tx 

	movlw b'00000000'			;Adr.: 0x06, L-byte
	call i2c_tx 

	movlw b'01000010'			;Adr.: 0x07, H-byte
	call i2c_tx 

	movlw b'00000010'			;Adr.: 0x07, L-byte
	call i2c_tx 

	call i2c_off			; Bus freigeben 
	return



;*******************************************************************************************
;	Unterprogramme I2C
;*******************************************************************************************
; I2C-Bus im Master-Mode bernehmen 
i2c_on
	bsf SSPCON,5				;Enable interne serielle Schnittstelle bit 5 = SSPEN
	bcf PIR1, SSPIF				; SSPIF Bit lschen 
 	bsf STATUS, RP0 
	bsf SSPCON2, SEN			; Bus bernahme anweisen 
	bcf	STATUS, RP0 
	goto i2c_warte 

; Wiederholte Startbedingung ausgeben
i2c_repeat
	bcf PIR1, SSPIF				; SSPIF Bit lschen 
 	bsf STATUS, RP0 
	bsf SSPCON2, RSEN			; Bus bernahme anweisen 
	bcf	STATUS, RP0 
	goto i2c_warte 

; ein Byte aus W senden 
i2c_tx 
	movwf SSPBUF				; -> zum I2C-Slave bertragen 
	goto i2c_warte 

;ein Byte vom Slave empfangen (nach SSPBUF) 
i2c_rx 
	bsf STATUS, RP0 
	bsf SSPCON2, RCEN			; Daten Empfang einschalten 
	bcf STATUS, RP0 
	goto i2c_warte 

; I2C-Bus Acknowledge Sequenz ausgeben (Lese Modus)
i2c_ack
	bcf PIR1, SSPIF				; SSPIF Bit lschen 
	bsf STATUS, RP0 
	bcf SSPCON2, ACKDT			;Acknowledge bit auf 0 setzen (Acknowledge)
	bsf SSPCON2, ACKEN			;Acknowledge Sequenz ausgeben (ACKDT wird ausgegeben)
	bcf STATUS, RP0 
	goto i2c_warte 

; I2C-Bus No Acknowledge Sequenz ausgeben (Lese Modus)
i2c_no_ack
	bcf PIR1, SSPIF				; SSPIF Bit lschen 
	bsf STATUS, RP0 
	bsf SSPCON2, ACKDT			;Acknowledge bit auf 1 setzen (not Acknowledge)
	bsf SSPCON2, ACKEN			;Acknowledge Sequenz ausgeben (ACKDT wird ausgegeben)
	bcf STATUS, RP0 
	goto i2c_warte 

; I2C-Bus wieder freigeben 
i2c_off 
	bsf STATUS, RP0 
	bsf SSPCON2, PEN			; Bus Freigabe anweisen 
	bcf STATUS, RP0 

i2c_warte 
	btfss PIR1, SSPIF			; fertig? 
	goto i2c_warte				; nein, noch nicht 
	bcf PIR1, SSPIF				; ja, alles fertig, nun noch SSPIF zurcksetzen 
	return 

;*******************************************************************************************
; Division eines 24 bit Wertes durch einen 16 bit Wert das Ergebnis wird in die
; Register des Zhlers gespeichert.
;
; Zhler / Nenner = Zhler + Rest 
; TEMP_13:11 / TEMP_16:15 = TEMP_13:11, Rest = TEMP_18:17
;
;*******************************************************************************************
div16
	clrf TEMP_18			;Rest von vorherigen Berechnungen lschen, TEMP_18/17=H/L
	clrf TEMP_17					
	movlw d'24'				;LOOP_1 mit 24 laden, da 24 bit im Zhler
	movwf LOOP_1
div16_loop
	rlf TEMP_11,F       	;den gesamtem Zhler um 1 nach links schieben, TEMP_13/12/11 = H/M/L
	rlf TEMP_12,F
	rlf TEMP_13,F

	rlf TEMP_17,F			;das Register fr den Rest um 1 nach links schieben
	rlf TEMP_18,F

	rlf LOOP_1,F			;das Carrybit wird vorbergehend im LOOP_1 gespeichert
									;dadurch kann man sich ein Register sparen	
	movf TEMP_15,W			;Nenner_L
	subwf TEMP_17,F			;Subtrahiere Nenner_L von Rest_L (REST_L-NENNER_L=REST_L)
	movf TEMP_16,W			;Nenner_H
	btfss STATUS, C
	incfsz TEMP_16,W		;erhhe Nenner_H
	subwf TEMP_18,W			;Subtrahiere Nenner_H von Rest_H (REST_H-NENNER_H=W-Reg)
	
	btfsc STATUS, C			;prfe, ob das Ergebnis positiv ist
	bsf LOOP_1,0			;wenn ja, setzte gespeichertes Carrybit in LOOP_1
	
	btfsc LOOP_1,0			;prfe das gespeicherte Carrybit in DIV_COUNTER
	goto div16_jump         ;wenn Carry=1 springe nach div16_jump
	
	movf TEMP_15,W			;Nenner_L,	Subtraktion wieder rckgngig machen, wenn das Ergebnis
	addwf TEMP_17,F			;REST_L,	negativ war und ein bertrag aufgetreten ist
	movf TEMP_18,W			;REST_H

div16_jump
	movwf TEMP_18			;Speichere den Rest vom W-Register in REST_H
	bcf STATUS,C			;Carrybit lschen und das gesicherte Carrybit in Register
	rrf LOOP_1,F			;LOOP_1 wieder zurcksichern.
	decfsz LOOP_1,F 		;LOOP_1 um 1 verringern
	goto div16_loop         ;wenn der Counter nicht null ist wird die nchste
	 
	rlf TEMP_11,F			;ZAEHLER_L		gesamter Zhler noch einmal nach links schieben, damit
	rlf TEMP_12,F			;ZAEHLER_M		das richtige Ergebnis im Zhler steht
	rlf TEMP_13,F			;ZAEHLER_H
	return
	
;*******************************************************************************************
; 16 bit Multiplikation
; Die Werte fr die Multiplikation mssen in den Registern TEMP_16:15
; und TEMP_18:17 stehen, das Ergebnis steht dann in den Registern TEMP_14:11
;*******************************************************************************************
mul16
	clrf TEMP_14			;Lschen des vorherigen Ergebnisses TEMP_14 =bit24-31
	clrf TEMP_13			;TEMP_13 =bit16-23
	clrf TEMP_12			;TEMP_12 =bit8-15
	clrf TEMP_11			;TEMP_11 =bit0-7
	bsf TEMP_12,7			;um zu erkennen, wann 16 mal geschoben wurde
mul16_loop
	rrf TEMP_16,F			;verschiebe High Byte Multiplikant1 um 1 nach rechts
	rrf TEMP_15,F			;verschiebe Low Byte Multiplikant1 um 1 nach rechts, wenn das LSB eine 1 ist wird das Carry gesetzt und es muss
	btfss STATUS,C			;eine Addition durchgefhrt werden ist das Carry = 0 entspricht
	goto mul16_rotate		;dies einer Multiplikation mit 0 und es wird nichts addiert
	movf TEMP_17,W			;Beginn der 16 bit Addition: Hole Low Byte Multiplikant2 ins W-Register
	addwf TEMP_13,F			;addiere bits 16-23 des Ergebnis Registers
	movf TEMP_18,W			;Hole High Byte Multiplikant2 ins W-Register
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz TEMP_18,W		;es ist ein bertrag vorhanden und das Highbyte von Multiplikant2 wird um 1 erhht
	addwf TEMP_14,F			;kein bertrag, das Highbyte wird addiert
mul16_rotate
	rrf TEMP_14,F			;Das 32 Bit Ergebnisregister wird um 1 nach rechts geschoben
	rrf TEMP_13,F
	rrf TEMP_12,F
	rrf TEMP_11,F
	btfss STATUS,C			;ist das Bit 7 von MUL_E_2 im Carry angekommen wurde das Ergebnis
	goto mul16_loop			;insgesamt 16 mal rotiert und die Berechnung ist beendet

	return                  ; fertig

;*******************************************************************************************
; 16 bit Addition:
; TEMP_13:11 + TEMP_18:17 = TEMP_13:11
;*******************************************************************************************

add16
	movf TEMP_17,W			;Hole Low Byte Zahl2 ins W-Register
	addwf TEMP_11,F			;addiere Low Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz TEMP_12,F		;es ist ein bertrag vorhanden und das L+1 Byte von Zahl2 wird um 1 erhht
	goto $+2				;kein bertrag beim L+1 Byte von Zahl2, 
	incf TEMP_13,F			;es ist ein bertrag vorhanden und das L+2 Byte von Zahl2 wird um 1 erhht

	movf TEMP_18,W			;Hole L+1 Byte Zahl2 ins W-Register
	addwf TEMP_12,F			;addiere L+1 Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incf TEMP_13,F			;es ist ein bertrag vorhanden und das L+2 Byte von Zahl2 wird um 1 erhht

	return

;*******************************************************************************************
; 16 bit Subtraktion:
; TEMP_13:11 - TEMP_18:17 = TEMP_13:11
;*******************************************************************************************

sub16
	movfw TEMP_17			;Hole 0/7-Byte von B nach W
	subwf TEMP_11,F			;Subtrahiere B von C, 0/7-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub16_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 8/15-Byte
	subwf TEMP_12,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub16_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 16/23-Byte
	subwf TEMP_13,F

sub16_2
	movfw TEMP_18			;Hole 8/15-Byte von B nach W
	subwf TEMP_12,F			;Subtrahiere B von C, 8/15-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	return					;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 16/23-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 16/23-Byte
	subwf TEMP_13,F

	return

;*******************************************************************************************
;	Unterprogramme zum Empfang/Auswertung des IR-Empfngers
;*******************************************************************************************

ir_routine
	clrf IR_ADRESSE			;Lsche Register
	clrf IR_BEFEHL
	bcf STAT_RADIO,2		;lsche IR-Fehlerbit
	bsf STAT_RADIO,1		;setze Statusbit fr alten Zustand RB0
	movlw d'14'				;14 Bits sollen empfangen werden
	movwf IR_BIT			;Zhler fr bertragene Bits
	clrf TMR2				;lsche Timer 2
	bcf PIR1,TMR2IF			;lsche Timer2 berlauf Flag

	movlw d'207'			;lade 222 (=>444us) in PR2 Register (222-14 Zeitversatz)
	bsf STATUS,RP0			;Select BANK_1
	movwf PR2
	bcf STATUS,RP0			;Select BANK_0
	btfss PIR1,TMR2IF		;prfen, ob Timer 2 bergelaufen ist (bei 162)
	goto $-1				;noch nicht bergelaufen
	bcf PIR1,TMR2IF			;lsche Timer2 berlauf Flag
ir_rout_loop
	bcf STATUS,C			;lsche Carry Bit, damit es beim rotieren nicht strt
	rlf IR_BEFEHL			;schiebe IR Befehl und IR Adresse nach links
	rlf IR_ADRESSE
	btfss IN_IR				;prfen, ob RB0 high ist, Bit Wert 1.Hlfte
	bsf IR_BEFEHL,0			;RB0 ist low, setze Bit 0 in Register IR Befehl
							;Prfung, ob positive oder negative Flanke vorhanden. Wenn nicht = Fehler:
	btfss IN_IR				;prfen, ob RB0 high ist, Bit Wert 1.Hlfte
	goto $+4				;RB0 ist low, weiter
	btfsc STAT_RADIO,1		;RB0 ist high, prfen ob alter Status von RB0 low war (=>STAT_RADIO,Bit1)
	goto ir_fehler			;alter Status von RB0 war auch high => Fehler
	goto $+3				;alter Status von RB0 war low => OK, weiter
	btfss STAT_RADIO,1		;RB0 ist low, prfen ob alter Status von RB0 high war (=>STAT_RADIO,Bit1)
	goto ir_fehler			;alter Status von RB0 war auch low => Fehler

	movlw 0xFF				;Timer 2 auf 890us einstellen (1x 256 + 1x 189)
	bsf STATUS,RP0			;Select BANK_1
	movwf PR2
	bcf STATUS,RP0			;Select BANK_0
	btfss PIR1,TMR2IF		;Abfrage, ob Timer2 bergelaufen ist
	goto $-1				;noch nicht bergelaufen
	bcf PIR1,TMR2IF			;lsche Timer2 berlauf Flag
	movlw d'188'			;Timer 2 auf 890us einstellen (1x 256 + 1x 189)
	bsf STATUS,RP0			;Select BANK_1
	movwf PR2
	bcf STATUS,RP0			;Select BANK_0
	btfss PIR1,TMR2IF		;Abfrage, ob Timer2 bergelaufen ist
	goto $-1				;noch nicht bergelaufen
	bcf PIR1,TMR2IF			;lsche Timer2 berlauf Flag
							
	bcf STAT_RADIO,1		;sichern des Status von RB0
	btfsc IN_IR				;prfen, ob RB0 low ist, Bit Wert 2.Hlfte
	bsf STAT_RADIO,1

	movlw d'187'			;Timer 2 auf 890us einstellen (1x 256 + 1x 189)
	bsf STATUS,RP0			;Select BANK_1
	movwf PR2
	bcf STATUS,RP0			;Select BANK_0
	btfss PIR1,TMR2IF		;Abfrage, ob Timer2 bergelaufen ist
	goto $-1				;noch nicht bergelaufen
	bcf PIR1,TMR2IF			;lsche Timer2 berlauf Flag
	movlw 0xFF				;Timer 2 auf 888us einstellen (1x 256 + 1x 188)
	bsf STATUS,RP0			;Select BANK_1
	movwf PR2
	bcf STATUS,RP0			;Select BANK_0
	btfss PIR1,TMR2IF		;Abfrage, ob Timer2 bergelaufen ist
	goto $-1				;noch nicht bergelaufen
	bcf PIR1,TMR2IF			;lsche Timer2 berlauf Flag

	decfsz IR_BIT			;1778 us abgelaufen, reduziere Bit Zhler und prfe ob null
	goto ir_rout_loop		;Zhler noch nicht null, zurck zur Schleife

	bcf STATUS,C			;lsche Carry Bit, damit es beim rotieren nicht strt
	rlf IR_BEFEHL			;schiebe IR Befehl und IR Adresse 2x nach links
	rlf IR_ADRESSE
	rlf IR_BEFEHL			;
	rlf IR_ADRESSE
	
	btfss IR_ADRESSE,7		;prfen, ob Startbits (Bits 7:6) =1, wenn nicht => Fehler
	goto ir_fehler
	btfss IR_ADRESSE,6		;
	goto ir_fehler
	bcf IR_ADRESSE,7		;lsche Startbits und Toggle Bit
	bcf IR_ADRESSE,6
	bcf IR_ADRESSE,5
	bcf STATUS,C			;lsche Carry Bit, damit es beim rotieren nicht strt
	rrf IR_BEFEHL			;
	rrf IR_BEFEHL			;
	bsf STAT_RADIO,1		;keine Fehler, setze Statusbit fr neuen IR Befehl
	btfss PIR1,TMR2IF		;Abfrage, ob Timer2 bergelaufen ist
	goto $-1				;noch nicht bergelaufen
	bcf PIR1,TMR2IF			;lsche Timer2 berlauf Flag
	return

ir_fehler
	bcf STAT_RADIO,1
	bsf STAT_RADIO,2		;Fehler, setze Statusbit fr fehlerhaften IR Befehl
	movlw d'59'				;30 ms Warttezeit nach einem Fehler (512us x 59 = 30208us => 30ms
	movwf IR_BIT
	btfss PIR1,TMR2IF		;Abfrage, ob Timer2 bergelaufen ist
	goto $-1				;noch nicht bergelaufen
	bcf PIR1,TMR2IF			;lsche Timer2 berlauf Flag
	decfsz IR_BIT			;512 us abgelaufen, reduziere Bit Zhler und prfe ob null
	goto $-4
	return


	ORG 0x07F0

cursor_tab				;Tabellen fr Cursorposition in den Menpunkten
	addwf PCL,f
	retlw 0x80			;DP 0: nicht belegt
	retlw 0x85			;DP 1: Menpunkt 4: Anzeige Radiotext ein/aus
	retlw 0x8D			;DP 2: Menpunkt 4: Anzeige Datum/Uhrzeit ein/aus
	retlw 0xC5			;DP 3: Menpunkt 4: Qmin fr RDS Anzeige
	retlw 0x86			;DP 4: Menpunkt 7: IR-Adresse 1
	retlw 0x8E			;DP 5: Menpunkt 7: IR-Adresse 2
	retlw 0xC6			;DP 6: Menpunkt 7: IR-Befehl Men
	retlw 0xCE			;DP 7: Menpunkt 7: IR-Befehl 0k
	retlw 0x85			;DP 8: Menpunkt 3: Volume
	retlw 0x8D			;DP 9: Menpunkt 3: Sender Nr.
	retlw 0xC6			;DP 10: Menpunkt 3: Standby 0/1
	retlw 0xCC			;DP 11: Menpunkt 3: Amp (Endstufe) 0/1

;*******************************************************************************************
;	PAGE 1, 	bsf PCLATH,3				;Select Page 1
;				bcf PCLATH,4
;*******************************************************************************************
	ORG 0x800					;PAGE 1

disp_menue_1
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse	
	movfw MENU_COUNT		;Schreiben der Menpunkt Nr.
	addlw 0x30				;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	movlw A'.'				;ASCII:
	call disp_daten			;bertrage das Zeichen

	movlw 0x08				;Programmzhler fuer Menpunkt tabelle im Bereich 0x0800..0x08FF 
	movwf PCLATH
	movfw MENU_COUNT
	addwf PCL,f
	nop
	goto dispm1_seek		;Menpunkt 1
	goto dispm1_savdel		;Menpunkt 2
	goto dispm1_start		;Menpunkt 3
	goto dispm1_rds_anzeige	;Menpunkt 4
	goto dispm1_ir_test		;Menpunkt 5
	goto dispm1_ir_adr		;Menpunkt 6
	goto dispm1_contrast	;Menpunkt 7

dispm1_seek					;Menpunkt 1
	movlw A'S'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'u'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'c'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'h'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'l'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'u'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'f'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto dispm1_end

dispm1_savdel				;Menpunkt 2
	movlw A'S'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'p'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'i'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'c'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'h'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'/'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw 0xC2				;Zeichen 3 in der 2. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'L'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'o'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A's'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'c'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'h'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto dispm1_end

dispm1_start				;Menpunkt 3
	movlw A'S'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto dispm1_end

dispm1_rds_anzeige			;Menpunkt 4
	movlw A'R'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'D'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'S'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'A'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'z'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'i'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'g'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto dispm1_end

dispm1_ir_test				;Menpunkt 5
	movlw A'I'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'R'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'T'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A's'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto dispm1_end

dispm1_ir_adr				;Menpunkt 6
	movlw A'I'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'R'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'A'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'd'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'.'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'/'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'B'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'f'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'.'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto dispm1_end

dispm1_contrast				;Menpunkt 7
	movlw A'K'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'o'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A's'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto dispm1_end

dispm1_end
	movlw d'250'			;Wartezeit von 250ms
	call wait 	
	bcf MENU_STAT,7			;lsche das Status Bit
	return

disp_radio
	movlw 0x80				;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'F'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'M'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'T'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'u'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'R'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'D'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'A'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'5'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'8'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'0'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'7'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'M'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	return

suche_start					;Menpunkt 1: Abfrage Suche starten?
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'S'				;ASCII:S
	call disp_daten			;bertrage das Zeichen
	movlw A'u'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'c'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'h'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A's'				;ASCII:S
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'?'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	call disp_ja_nein		;Unterprogramm zum Schreiben von "Nein=Menue Ja=OK"
	return

disp_save_delete			;Menpunkt 2: Abfrage Speichern?/Lschen?
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	btfss STAT_RADIO,0		;prfen, ob bit fr Frequenz gesetzt ist
	goto disp_del_pgm		;bit ist nicht gesetzt, weiter bei Lschen programm

	call disp_frequenz		;Unterprogramm zur Anzeige der Frequenz auf dem Display
	movlw A'S'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'p'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'i'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'c'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'h'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'?'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_ja_nein		;Unterprogramm zum Schreiben von "Nein=Menue Ja=OK"
	return
disp_del_pgm
	movlw A'F'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'M'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw SENDER			;hole die Sendernummer nach W
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'L'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'o'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A's'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'c'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'h'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'?'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_ja_nein		;Unterprogramm zum Schreiben von "Nein=Menue Ja=OK"
	return
	

disp_ja_nein				;Unterprogramm zum Schreiben von "Nein=Menue Ja=OK"
	movlw 0xC0				;Zeichen 1 in der 2. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'N'				;ASCII:S
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'i'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'='				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'M'				;ASCII:S
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'u'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'J'				;ASCII:S
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'='				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'O'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw A'K'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	return

disp_menu_seek
	movlw 0x80				;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'P'				;
	call disp_daten			;bertrage das Zeichen
	movfw SENDER			;hole die Sendernummer nach W
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	call disp_frequenz		;Unterprogramm zur Anzeige der Frequenz auf dem Display
	movlw A'Q'				;ASCII: Q
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII: :
	call disp_daten			;bertrage das Zeichen
	;bcf RB0BH_L,0			;lsche Bit0
	movfw RB0BH_H			;kopiere das Qualittssignal (0...255) in das Multiplikationsregister Temp_16:15
	movwf TEMP_15			; Qualitt = Signal x 100/256
	clrf TEMP_16
	movlw 0x64				;kopiere Wert 100 (hex 0x64) in 2tes Multiplikationsregister Temp_18:17
	movwf TEMP_17
	clrf TEMP_18
	bcf PCLATH,3			;Select Page 0
	call mul16				;Multiplikation
	bsf PCLATH,3			;Select Page 1	
	movfw TEMP_12
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display
	movlw A'%'				;ASCII: %
	call disp_daten			;bertrage das Zeichen
	movlw A'1'				;Nur fr Test!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	btfss RB0BH_H,0
	movlw A'0'	
	call disp_daten			;bertrage das Zeichen
	return

disp_sp_voll
	call clear_display		;Unterprogramm zum lschen des Displays
	movlw 0x81				;Zeichen 2 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'S'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'p'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'i'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'c'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'h'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'v'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'o'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'l'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'l'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	return
	

disp_set_irtest				;Menpunkt 6
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'A'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'd'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	btfss STAT_RADIO,2		;prfen, ob IR Fehlerbit gesetzt ist
	goto $+3				;Fehlerbit nicht gesetzt, weiter
	call disp_irff			;Fehlerbit gesetzt, Anzeige FF
	goto disp_set_irtest2	;weiter Zeile 2
	movfw IR_ADRESSE		;Daten fr RC5 IR-Adresse
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display
disp_set_irtest2
	movlw 0xC0				;Zeichen 1 in der 2. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'B'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'f'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	btfss STAT_RADIO,2		;prfen, ob IR Fehlerbit gesetzt ist
	goto $+3				;Fehlerbit nicht gesetzt, weiter
	call disp_irff			;Fehlerbit gesetzt, Anzeige FF
	goto disp_irtest_end	;weiter bei Ende
	movfw IR_BEFEHL			;Daten fr RC5 IR-Befehl
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display
disp_irtest_end
	movlw d'250'			;Wartezeit von 250ms
	call wait 		
	bcf STAT_RADIO,2		;lsche IR-Fehlerbit
	bcf STAT_RADIO,1		;lsche Statusbit
	return

disp_irff
	movlw A'F'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'F'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_set_start				;Menpunkt 3
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'V'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_17				;Daten fr Volume
	call convdisp_hex_2dig		;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display	
	call disp_blank				;Unterprogramm zum Schreiben von "blank"
	call disp_blank				;Unterprogramm zum Schreiben von "blank"
	movlw A'P'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_18				;Daten fr Sender Nr.
	call convdisp_hex_2dig		;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display	
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'b'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'y'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x31
	btfss TEMP_11,5				;Bit fr Standby
	movlw 0x30
	call disp_daten				;bertrage das Zeichen
	call disp_blank				;Unterprogramm zum Schreiben von "blank"
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'p'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x31
	btfss TEMP_11,4				;Bit fr Standby
	movlw 0x30
	call disp_daten				;bertrage das Zeichen
	return

disp_set_rdsanz				;Menpunkt 4
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'x'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x31
	btfss TEMP_11,7				;Bit fr Anzeige Radiotext
	movlw 0x30
	call disp_daten				;bertrage das Zeichen
	movlw 0x02					;"blank" soll 2x geschrieben werden
	call disp_blank_x			;Unterprogramm zum x mal Schreiben von "blank"
	movlw A'Z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x31
	btfss TEMP_11,6				;Bit fr Anzeige Datum/Uhrzeit
	movlw 0x30
	call disp_daten				;bertrage das Zeichen

	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'Q'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_12				;Daten fr Qmin
	call convdisp_hex_2dig		;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display	
	movlw A'%'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_set_iradr				;Menpunkt 7
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw IR_ADR_1				;daten fr 1.RC5-Adresse
	call convdisp_hex_2dig		;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display
	call disp_blank				;Unterprogramm zum Schreiben von "blank"
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw IR_ADR_2				;daten fr 2.RC5-Adresse
	call convdisp_hex_2dig		;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display

	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'M'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw IR_BEF_MEN			;daten fr RC5-Befehl Men
	call convdisp_hex_2dig		;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display
	call disp_blank				;Unterprogramm zum Schreiben von "blank"
	movlw A'O'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'K'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x02					;"blank" soll 2x geschrieben werden
	call disp_blank_x			;Unterprogramm zum x mal Schreiben von "blank"
	movfw IR_BEF_OK				;daten fr RC5-Befehl OK
	call convdisp_hex_2dig		;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display
	return

disp_set_contrast			;Menpunkt 8
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'K'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'o'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A's'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw CONTRAST			;Wert fr Kontrast
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display
	return


disp_p_freq_q				;Anzeige von Programm Nr., Frequenz und Qualitt
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'F'				;
	call disp_daten			;bertrage das Zeichen
	movlw A'M'				;
	call disp_daten			;bertrage das Zeichen
	btfss STAT_RADIO,0		;prfen, ob Bit fr manuelle Frequenzeinstellung gesetzt ist
	goto $+6				;Bit nicht gesetzt, schreibe die Sender Nr.
	movlw 0x20				;Bit gesetzt, ersetze die Sender Nr. durch "blank": Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	goto $+3
	movfw SENDER			;hole die Sendernummer nach W
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	call disp_frequenz		;Unterprogramm zur Anzeige der Frequenz auf dem Display

	btfsc STAT_TIMER,3
	goto disp_volume1		;Bit 3 ist gesetzt, weiter bei disp_volume
	btfsc STAT_TIMER,4
	goto disp_volume1		;Bit 4 ist gesetzt, weiter bei disp_volume

	movlw A'Q'				;ASCII: Q
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII: :
	call disp_daten			;bertrage das Zeichen
	bcf RB0BH_L,0			;lsche Bit0
	movfw RB0BH_H			;kopiere das Qualittssignal (0...255) in das Multiplikationsregister Temp_16:15
	movwf TEMP_15			; Qualitt = Signal x 100/256
	clrf TEMP_16
	movlw 0x64				;kopiere Wert 100 (hex 0x64) in 2tes Multiplikationsregister Temp_18:17
	movwf TEMP_17
	clrf TEMP_18
	bcf PCLATH,3			;Select Page 0
	call mul16				;Multiplikation
	bsf PCLATH,3			;Select Page 1	
	movfw TEMP_12
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display
	movlw A'%'				;ASCII: %
	call disp_daten			;bertrage das Zeichen
	return

disp_volume1
	btfsc STAT_TIMER,3
	goto disp_volume
	bsf STAT_TIMER,3
	bcf STAT_TIMER,4
disp_volume
	movlw 0x8B				;Zeichen 12 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'V'				;ASCII: Q
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII: :
	call disp_daten			;bertrage das Zeichen
	bcf WB05H_L,7			;lsche Bit 7
	movfw WB05H_L			;hole Volumenwert nach W (0...15)
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display
	bsf WB05H_L,7			;Bit 7 wieder setzen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	return	

disp_rds_daten				;Anzeige von Station, Radiotext und Datum/Uhrzeit	
	movlw 0xC0				;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	btfsc STAT_DISP,0		;prfen, ob Bit fr Anzeige Datum/Uhrzeit nicht gesetzt ist
	goto disp_dat_uz		;Bit ist gesetzt, weiter bei Anzeige Datum/Uhrzeit
	btfsc STAT_DISP,2		;prfen, ob Bit fr Anzeige Radiotext nicht gesetzt ist
	goto disp_text			;Bit ist gesetzt, weiter bei Anzeige Radiotext

disp_station
	movlw d'4'				;"blank" soll 4x geschrieben werden
	call disp_blank_x
	movlw d'8'				;8 Zeichen sollen bertragen werden
	movwf LOOP_1			;	
	bcf STATUS,IRP			;lsche IRP Bit
	movlw 0xE0
	movwf FSR				;schreibe die Adresse ins FSR Register
	movfw INDF				;hole die daten der Adresse nach W
	call disp_daten			;bertrage das Zeichen
	incf FSR				;erhhe die Adresse um 1
	decfsz LOOP_1			;reduziere den Zhler fr die bertragenen Zeichen un prfe ob Null
	goto $-4				;Zhler noch nicht 0, zurck zur Schleife
	btfss STAT_TIMER,2		;prfen, ob 4s Timer Bit gesetzt ist
	goto $+4				;4 Sek noch nicht um, zurck zur Hauptschleife
	bcf STAT_TIMER,2		;lsche Bit fr 4s Timer	
	bsf STAT_DISP,2			;setze Bit fr Anzeige Radiotext
	bcf STAT_DISP,3			;lsche Bit fr Radiotext 2
	movlw d'4'				;"blank" soll 4x geschrieben werden
	call disp_blank_x
	return

disp_text
	btfss STAT_DISP,7		;prfen, ob Radiotext angezeigt werden soll, d.h. Bit 7 gesetzt ist
	goto disp_text0			;Bit 7 ist nicht gesetzt, weiter bei Anzeige Datum/Uhrzeit
	movlw d'16'				;16 Zeichen sollen bertragen werden
	movwf LOOP_1			;

	bcf STATUS,IRP			;lsche IRP Bit fr Anzeige Register 0xA0 bis 0xDF
	btfsc STAT_DISP,3		;Prfen, ob Bit fr Anzeige Register 0x120 bis 0x159 nicht gesetzt ist
	bsf STATUS,IRP			;Bit ist gesetzt, setze IRP Bit fr Anzeige Register 0x120 bis 0x159

	movfw TEXT_ADR			;im ersten Schritt wird geprft, ob die Werte =0. wenn ja, Abbruch
	movwf FSR				;schreibe die Adresse ins FSR Register
	movfw INDF				;hole den Wert der Adresse nach W
	btfss STATUS,Z			;prfen ob der Wert =0
	goto disp_text1			;Wert >0<, weiter
	incf FSR				;erhhe die Adresse um 1
	decfsz LOOP_1			;reduziere den Zhler fr die bertragenen Zeichen un prfe ob Null
	goto $-5				;Zhler noch nicht 0, zurck zur Schleife
							;Zhler =0 und alle Werte =0
	btfss STAT_DISP,3		;Prfen, ob Bit fr Anzeige Register 0x1A0 bis 0x1D9 gesetzt ist
	goto $+3				;Bit ist nicht gesetzt, setze Bit
	bcf STAT_DISP,3			;lsche Bit fr Anzeige Register 0x1A0 bis 0x1D9
	goto $+2
	bsf STAT_DISP,3			;setze Bit fr Anzeige Register 0x1A0 bis 0x1D9
disp_text0
	bcf STAT_DISP,2			;lsche Bit fr Text
	bsf STAT_DISP,0			;setze Bit fr Datum/Uhrzeit
	movlw 0xA0				;Rcksetzen auf Anfang des 1.Textblocks
	movwf TEXT_ADR
	goto disp_dat_uz		;weiter bei Datum
disp_text1
	movlw d'16'				;16 Zeichen sollen bertragen werden
	movwf LOOP_1			;	
	movfw TEXT_ADR
	movwf FSR				;schreibe die Adresse ins FSR Register
	movlw d'13'				;hole Wert 13 nach W (Wert 13 markiert das Ende einer Textbertragung)
	subwf INDF,w			; und subtrahiere INDF in W
	btfsc STATUS,Z			;prfen, ob >0<
	goto disp_text_end1		;INDF war 13, die restlichen Digits werden als "blank" geschrieben
	movfw INDF				;hole die daten der Adresse nach W
	call disp_daten			;bertrage das Zeichen
	incf FSR				;erhhe die Adresse um 1
	decfsz LOOP_1			;reduziere den Zhler fr die bertragenen Zeichen un prfe ob Null
	goto $-8				;Zhler noch nicht 0, zurck zur Schleife
disp_text_end
	btfss STAT_TIMER,2
	return
	bcf STAT_TIMER,2		;lsche Bit fr 4s Timer
	movlw 0xD0
	subwf TEXT_ADR,w
	btfsc STATUS,Z			;
	goto disp_text_end2
	movlw d'16'
	addwf TEXT_ADR,f
	return
disp_text_end1
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	decfsz LOOP_1			;reduziere den Zhler fr die bertragenen Zeichen un prfe ob Null
	goto $-2
	btfss STAT_TIMER,2
	return
	bcf STAT_TIMER,2		;lsche Bit fr 4s Timer
disp_text_end2
	movlw 0xA0				;Rcksetzen auf Anfang des 1.Textblocks
	movwf TEXT_ADR
	btfsc STAT_DISP,3		;Prfen, ob Bit fr Anzeige Register 0x1A0 bis 0x1D9 nicht gesetzt ist
	goto $+3
	bsf STAT_DISP,3
	return

	bcf STAT_DISP,3
	bcf STAT_DISP,2
	bsf STAT_DISP,0
	return

disp_dat_uz
	btfss STAT_DISP,6		;prfen, ob Datum/Uhrzeit angezeigt werden sollen, d.h. Bit 6 gesetzt ist
	goto $+3
	btfsc STAT_RADIO,7		;prfen, ob Bit fr Freigabe Anzeige Datum/Uhrzeit nicht gesetzt ist
	goto $+3				;Bit ist gesetzt
	bcf STAT_DISP,0			;STAT_DISP Bit 6 bzw STAT_RADIO Bit 7 ist nicht gesetzt, lsche Bit fr Anzeige Datum/Uhrzeit
	goto disp_station		;weiter bei Anzeige des Senders

	btfsc STAT_DISP,1		;prfen, ob Bit fr Anzeige Uhrzeit gesetzt ist
	goto disp_uz			;Bit ist gesetzt, weiter bei Anzeige Uhrzeit
							;-SA-01.01.2000--
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	movlw 0x0F				;Programmzhler fuer Wochentagstabelle im Bereich 0x0FF0..0x0FFF 
	movwf PCLATH			;
	movfw W_TAG				;hole die Wochentagszahl nach W
	call tab_wt				;Unterprogramm zum holen der 2 Anfangsbuchstaben des Wochentags aus der Tabelle, 1.Buchstabe
	call disp_daten			;bertrage das Zeichen
	movfw W_TAG				;hole die Wochentagszahl nach W
	addlw d'7'				;addiere 7 (fr 2.Buchstabe)
	call tab_wt				;Unterprogramm zum holen der 2 Anfangsbuchstaben des Wochentags aus der Tabelle, 2.Buchstabe
	call disp_daten			;bertrage das Zeichen
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	movfw TAG
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display	
	movlw A'.'				;Zeichen fr "."
	call disp_daten			;bertrage das Zeichen
	movfw MONAT
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display	
	movlw A'.'				;Zeichen fr "."
	call disp_daten			;bertrage das Zeichen
	movlw A'2'				;Zeichen fr "2"
	call disp_daten			;bertrage das Zeichen
	movlw A'0'				;Zeichen fr "0"
	call disp_daten			;bertrage das Zeichen	
	movfw JAHR
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display	
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	btfss STAT_TIMER,2
	return
	bsf STAT_DISP,1			;setze Bit fr Anzeige Uhrzeit
	bcf STAT_TIMER,2		;lsche Bit fr 4s Timer
	return
disp_uz						;----00:00:00----
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	movfw STUNDE
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display	
	movlw A':'				;Zeichen fr ":"
	call disp_daten			;bertrage das Zeichen
	movfw MINUTE
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display	
	movlw A':'				;Zeichen fr ":"
	call disp_daten			;bertrage das Zeichen
	movfw SEKUNDE
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display		
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	btfss STAT_TIMER,2
	return
	bcf STAT_TIMER,2		;lsche Bit fr 4s Timer
	bcf STAT_DISP,0			;lsche Bit fr Anzeige Datum/Uhrzeit
	bcf STAT_DISP,1			;lsche Bit fr Anzeige Uhrzeit	
	return

disp_frequenz				;Unterprogramm zur Anzeige der Frequenz auf dem Display
	movlw 0x66				;Frequenz: kopiere Wert 870 (hex: 0x000366) in die Additionsregister TEMP_13:11
	movwf TEMP_11			;Frequenz: 87,0 + 00,0...25,5 => 87,0...113,5 MHz
	movlw 0x03
	movwf TEMP_12
	clrf TEMP_13
	movfw RB0AH_L			;kopiere die Frequenz in die Additionsregister TEMP_18:17
	movwf TEMP_17
	clrf TEMP_18
	bcf PCLATH,3			;Select Page 0	
	call add16
	bsf PCLATH,3			;Select Page 1	
	call convert_hex_dig	;Auftrennen einer 4-stelligen Zahl in die einzelnen Digits: z.B. 1234 in 1,2,3,4
	movfw TEMP_16			;1000er, untere 4 Bit
	addlw 0x30				;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	movfw TEMP_15			;100er, untere 4 Bit
	addlw 0x30				;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	movfw TEMP_14			;10er, untere 4 Bit
	addlw 0x30				;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	movlw A'.'				;ASCII: . fr Komma
	call disp_daten			;bertrage das Zeichen
	movfw TEMP_13			;1er, untere 4 Bit
	addlw 0x30				;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	return



disp_ir_zahl
	movfw RB0AH_L			;kopieren der 16 bit Zahl in Temp 12:11 Register
	movwf TEMP_11
	movfw RB0AH_H			;
	movwf TEMP_12
	call convert_hex_dig	;Unterprogramm zum Zerlegen einer 16-Bit Zahl in die einzelnen Digits
							; (max.9.999)zur Anzeige auf dem Display
	btfss STAT_RADIO,0		;prfen, ob Bit fr Frequenz gesetzt ist (4 Stellen)
	goto disp_ir_pgrm		;Bit ist nicht gesetzt => Anzeige Stations Nr.
	movlw 0x85				;Zeichen 6 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_16			;1000er, untere 4 Bit
	addlw 0x30				;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	movfw TEMP_15			;100er, untere 4 Bit
	addlw 0x30				;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	movfw TEMP_14			;10er, untere 4 Bit
	addlw 0x30				;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	movlw A'.'				;ASCII: . fr Komma
	call disp_daten			;bertrage das Zeichen
	movfw TEMP_13			;1er, untere 4 Bit
	addlw 0x30				;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	return

disp_ir_pgrm
	movlw 0x82				;Zeichen 3 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_14			;10er, untere 4 Bit
	addlw 0x30				;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	movfw TEMP_13			;1er, untere 4 Bit
	addlw 0x30				;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	return

clear_rdsdisp				;lsche Zeile 2
	movlw 0xC0				;Zeichen 1 in der 2. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw 0x10				;"blank" soll 16 mal geschrieben werden
	call disp_blank_x		;Schreiben von "blank" x mal entsprechend dem wert in W
	return

disp_blank					;Unterprogramm zum Schreiben von "blank"
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	return

disp_blank_x				;Schreiben von "blank" x mal entsprechend dem wert in W
	movwf LOOP_1			;kopiere W nach Loop_1
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	decfsz LOOP_1			;reduziere Zhler um 1 und prfe ob null
	goto $-3				;Zhler noch nicht Null, zurck zur Schleife
	return					;Zhler = Null, fertig


;*******************************************************************************************
;	Unterprogramme Initialisierung/Ansteuerung des Display
;*******************************************************************************************


Init_display					;Unterprogramm zur Initialisierung des Displays
	movlw d'250'				;geforderte Wartezeit des Displays von 200ms
	call wait 		
	movlw 0x38					;Befehl: Function Set
	call disp_befehl	;
	movlw 0x39					;Befehl: Function Set, (muss zweimal bertragen werden)
	call disp_befehl	;
	movlw 0x14					;Befehl: Bias Set
	call disp_befehl	;

	movfw CONTRAST				;kopiere Contrast in Shift Register
	movwf SHIFT_REG
	bcf SHIFT_REG,4				;lsche Bits 4+5, da hier nur Bits 0-3 bentigt werden
	bcf SHIFT_REG,5	
	movfw SHIFT_REG
	addlw 0x70					;Befehl: Contrast Set : addiere die oberen 4 Bits (01110000)
	call disp_befehl			;

	movlw 0x50					;kopiere obere 6 Bits (01010000) in Shift Register
	movwf SHIFT_REG
	btfsc CONTRAST,4			;prfen, ob Bit 4 nicht gesetzt ist
	bsf SHIFT_REG,0				;Bits 4 ist gesetzt, setze Bit 0 in Shift Register
	btfsc CONTRAST,5			;prfen, ob Bit 5 nicht gesetzt ist
	bsf SHIFT_REG,1				;Bits 5 ist gesetzt, setze Bit 1 in Shift Register
	movfw SHIFT_REG				;Befehl: Power/Icon/Contrast Contr.
	call disp_befehl			;

	movlw 0x69					;Befehl: Follower Control
	call disp_befehl	;
	movlw d'250'				;geforderte Wartezeit des Displays von 200ms
	call wait 		
	movlw 0x0C					;Befehl: Display On/Off Control (00001100)
	call disp_befehl	;
	call clear_display			;Aufruf des Unterprogramms zum Lschen des Displays
	movlw 0x06					;Befehl: Entry Mode Set (00000110)
	call disp_befehl	;
	return


clear_display					;Unterprogramm zum lschen des Displays					
	movlw 0x01					;Befehl: Clear Display
	call disp_befehl	;
	movlw d'7'					;geforderte Wartezeit des Displays von 7ms
	call wait 		
	return

cursor_off
	movlw 0x0C					;Befehl: Display On/Off Control, Cursor OFF (00001100)
	goto $+2
cursor_on
	movlw 0x0E					;Befehl: Display On/Off Control, Cursor ON (00001110)
	call disp_befehl			;
	movlw d'20'					;geforderte Wartezeit min 27us
	movwf LOOP_1				;
	decfsz LOOP_1				;
	goto $-1
	return

disp_befehl
	movwf SHIFT_REG				;Register mit den Daten, die ausgegeben werden sollen
	bcf DISP_EN					;Enable auf low setzen
	bcf DISP_RS					;RS auf Low setzen, da es sich um ein Kommando handelt
	call disp_routine			;Aufruf des Unterprogramms zum Schreiben der Display Adresse / Daten
	return

disp_daten
	movwf SHIFT_REG				;Das W-Register wird in das Reg. SHIFT_REG kopiert
	bcf DISP_EN					;Enable auf low setzen
	bsf DISP_RS					;RS auf High setzen, da es sich um ein Zeichen handelt
	call disp_routine			;Aufruf des Unterprogramms zum Schreiben der Display Adresse / Daten
	return

disp_routine
	bcf SI_CLK					;Takt auf Low setzen, damit die Daten bei der steigenden
									;Flanke bernommen werden knnen
	movlw d'8'					;es sollen 8 Bits bertragen werden
	movwf SHIFT_CNT				;Zhler fr die bertragenen bits
shiftDaten
	rlf SHIFT_REG,F				;Das Zeichen wird nach links verschoben. Das Carryflag wird
									;an den Ausgang weitergegeben
	btfsc STATUS,C				;Prfen, welchen Wert das Carryflag hat
	bsf SI_DAT					;Carryflag = 1 -> Pin SI_DAT auf Highpegel setzen
	btfss STATUS,C				;Nochmal prfen, da immer nur ein Befehl bersprungen wird
	bcf SI_DAT					;Carryflag = 0 -> Pin SI_DAT auf Lowpegel setzen
	nop							;kurze Wartezeit, um die bertragungsgeschwindigkeit zu reduzieren
	nop
	bsf SI_CLK					;mit dieser steigenden Flanke werden die Daten bernommen
	nop							;kurze Wartezeit
	nop
	bcf SI_CLK					;negative Flanke des Taktes
	decfsz SHIFT_CNT,F			;ein bit wurde bertragen -> Zhler um 1 verringern
	goto shiftDaten				;nchstes bit bertragen
	bsf DISP_EN					;Enable auf high setzen, mit der positiven Flanke werden Daten bertragen 
	bcf DISP_RS					;bertragung beendet Pin RS auf Low zurcksetzen
	bcf SI_DAT					;Pin SI_DAT auf Low zurcksetzen
	movlw d'30'					;geforderte Wartezeit (min. 26,3 us)
	movwf SHIFT_CNT
	decfsz SHIFT_CNT
	goto $-1
	return						;Rcksprung aus dem Unterprogramm


;*******************************************************************************************
;	Weitere Unterprogramme zur Anzeige auf dem Display
;*******************************************************************************************

convdisp_hex_2dig				;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return

convdisp_hex_3dig				;Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_13				;Daten 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return


convert_dig					;Unterprogramm zur Umwandlung von zB Zahl 123 in 1,2,3
	movwf TEMP_15			;Temp Register fr Zwischenspeicherung des zu zerlegenden Wertes
	clrf TEMP_13			;lsche Temp Register
	clrf TEMP_14			;lsche Temp Register
dig_100er
	movlw d'100'			;schreibe Wert 100 ins W-Register
	subwf TEMP_15,w			;reduziere den zu zerlegenden Werte um 100 im W-Register (zum Testen)
	btfss STATUS,C			;prfen,ob Ergebnis negativ
	goto dig_10er			;Ergebnis negativ, weiter bei 10er
	incf TEMP_13			;Ergebnis noch positiv, Erhhe 100er um 1 
	movlw d'100'			; und reduziere den zu zerlegenden Werte um 100 im F-Register
	subwf TEMP_15,f			;
	goto dig_100er			;Zurck zur Schleife
dig_10er
	movlw d'10'				;schreibe Wert 10 ins W-Register
	subwf TEMP_15,w			;reduziere den zu zerlegenden Werte um 10 im W-Register (zum Testen)
	btfss STATUS,C			;prfen,ob Ergebnis negativ
	goto dig_1er			;Ergebnis negativ, weiter bei 1er
	incf TEMP_14			;Ergebnis noch positiv, Erhhe 10er um 1 
	movlw d'10'				; und reduziere den zu zerlegenden Werte um 10 im F-Register
	subwf TEMP_15,f			;
	goto dig_10er			;Zurck zur Schleife
dig_1er
	movfw TEMP_15
	return

convert_hex_dig				;Unterprogramm zum Zerlegen einer 16-Bit Zahl in die einzelnen Digits (max.9.999)zur Anzeige auf dem Display
	clrf TEMP_13
	clrf TEMP_16			;Lsche Register, von der 16-Bit Zahl in Register Temp_12:11 wird Temp_18:17 abgezogen bis Ergebnis negativ
	movlw 0xE8				;Lade Temp_18:17 mit 1.000 (0x03E8)
	movwf TEMP_17
	movlw 0x03				;
	movwf TEMP_18
	incf TEMP_16			;erhhe Digit um 1
	bcf PCLATH,3			;Select Page 0
	call sub16				;Temp_12:11 minus Temp_18:17, das Ergebnis steht in Temp_12:11
	bsf PCLATH,3			;Select Page 1
	btfss TEMP_13,7			;Prfen Ergebnis negativ
	goto $-5				;Ergebnis noch positiv, zurck zur Schleife
	decf TEMP_16			;Ergebnis negativ, letzte Erhhung des Digit 
	bcf PCLATH,3			;Select Page 0
	call add16				;   und letzte Subtraktion rckgngig machen

	clrf TEMP_15			;Lsche Register, von der 16-Bit Zahl in Register Temp_12:11 wird Temp_18:17 abgezogen bis Ergebnis negativ
	movlw 0x64				;Lade Temp_18:17 mit 100 (0x0064)
	movwf TEMP_17
	clrf TEMP_18
	incf TEMP_15			;erhhe Digit um 1
	bcf PCLATH,3			;Select Page 0
	call sub16				;Temp_12:11 minus Temp_18:17, das Ergebnis steht in Temp_12:11
	bsf PCLATH,3			;Select Page 1
	btfss TEMP_13,7			;Prfen Ergebnis negativ
	goto $-5				;Ergebnis noch positiv, zurck zur Schleife
	bcf PCLATH,3			;Select Page 0
	decf TEMP_15			;Ergebnis negativ, letzte Erhhung des Digit 
	call add16				;   und letzte Subtraktion rckgngig machen

	clrf TEMP_14			;Lsche Register, von der 16-Bit Zahl in Register Temp_12:11 wird Temp_18:17 abgezogen bis Ergebnis negativ
	movlw 0x0A				;Lade Temp_18:17 mit 10 (0x0000000A)
	movwf TEMP_17
	clrf TEMP_18
	incf TEMP_14			;erhhe Digit um 1
	bcf PCLATH,3			;Select Page 0
	call sub16				;Temp_12:11 minus Temp_18:17, das Ergebnis steht in Temp_12:11
	bsf PCLATH,3			;Select Page 1
	btfss TEMP_13,7			;Prfen Ergebnis negativ
	goto $-5				;Ergebnis noch positiv, zurck zur Schleife
	bcf PCLATH,3			;Select Page 0
	decf TEMP_14			;Ergebnis negativ, letzte Erhhung des Digit 
	call add16				;   und letzte Subtraktion rckgngig machen

	bsf PCLATH,3			;Select Page 1
	movfw TEMP_11			;
	movwf TEMP_13
	return


wait
	movwf LOOP_1
loop_250
	movlw d'250'			;loop_10 mu 4 Takten entsprechen fr 1ms: 4MHz /4/250/4Takte
	movwf LOOP_2
loop_10
	nop
	decfsz LOOP_2
	goto loop_10
	decfsz LOOP_1
	goto loop_250
	return


	ORG 0x0FF0

tab_wt			;Tabelle Wochentag
	addwf PCL,f
	retlw A'S'	;Datenpunkt 0:	S amstag
	retlw A'S'	;Datenpunkt 1:	S onntag
	retlw A'M'	;Datenpunkt 2:	M ontag
	retlw A'D'	;Datenpunkt 3:	D ienstag
	retlw A'M'	;Datenpunkt 4:	M ittwoch
	retlw A'D'	;Datenpunkt 5:	D onnerstag
	retlw A'F'	;Datenpunkt 6:	F reitag

	retlw A'A'	;Datenpunkt 7:	S A mstag
	retlw A'O'	;Datenpunkt 8:	S O nntag
	retlw A'O'	;Datenpunkt 9:	M O ntag
	retlw A'I'	;Datenpunkt 10:	D I enstag
	retlw A'I'	;Datenpunkt 11:	M I ttwoch
	retlw A'O'	;Datenpunkt 12:	D O nnerstag
	retlw A'R'	;Datenpunkt 13:	F R eitag

	


;*******************************************************************************************
;	PAGE 2, 	bcf PCLATH,3				;Select Page 2
;				bsf PCLATH,4
;*******************************************************************************************
	ORG 0x1000					;PAGE 2	





	END                	; Ende des Programms

